import logging

from .models import Payment, PaymentsDetails, SatCFDI

logger = logging.getLogger(__name__)


def format_head(cfdi):
    return str(cfdi["TipoDeComprobante"]) + " " + cfdi["Version"] + "\n" + \
           str(cfdi.uuid) + "\n- " + cfdi.name


def format_head_pre(cfdi):
    return str(cfdi["TipoDeComprobante"]) + " " + cfdi["Version"] + "\n" + \
           "*" + cfdi["Serie"] + cfdi["Folio"]


def format_fecha(cfdi):
    return str(cfdi["Fecha"]) + "\n" + cfdi["LugarExpedicion"].code + "\n" + str(cfdi["Receptor"]["UsoCFDI"])


def format_emisor(cfdi):
    emisor = cfdi["Emisor"]
    return emisor["Nombre"] + "\n" + emisor["Rfc"] + "\n" + str(emisor["RegimenFiscal"])


def format_receptor(cfdi):
    receptor = cfdi["Receptor"]
    return receptor.get("Nombre", "") + "\n" + receptor["Rfc"] + " - " + receptor.get("DomicilioFiscalReceptor", "") + "\n" + str(receptor.get("RegimenFiscalReceptor", ""))


def format_forma_pago(cfdi):
    return str(cfdi.get("FormaPago")) + "\n" + \
           str(cfdi.get("MetodoPago")) + "\n" + \
           str(cfdi["Moneda"])


def format_forma_pago_dr(payment: Payment):
    if payment.pago is None:
        return format_forma_pago(payment.comprobante)

    return str(payment.pago["FormaDePagoP"]) + "\n" + \
           str(payment.docto_relacionado.get("MetodoDePagoDR")) + "\n" + \
           str(payment.docto_relacionado["MonedaDR"])


def format_fecha_pago(payment: PaymentsDetails):
    if payment.pago:
        return payment.pago["FechaPago"]

    def_pagado = payment.comprobante.saldo_pendiente == 0
    if def_pagado:
        return payment.comprobante["Fecha"]
    else:
        return "Pendiente"


def format_estado_cfdi(cfdi: SatCFDI):
    estatus = cfdi.consulta_estado()
    if not estatus:
        return ""

    return (
            (estatus["Estado"] or "") + "\n" +
            (estatus["EsCancelable"] or "") + "\n" +
            (estatus["EstatusCancelacion"] or "")
    )


def format_pagos(cfdi: SatCFDI):
    if cfdi["TipoDeComprobante"] == "P":
        for pago in cfdi["Complemento"]["Pagos"]["Pago"]:
            response = ""
            for doc_rel in pago.get("DoctoRelacionado", []):
                response += doc_rel["IdDocumento"] + '\n' + \
                            '- ' + doc_rel.get("Serie", "") + doc_rel.get("Folio", "") + '\n'
            return response[:-1]

    if cfdi["TipoDeComprobante"] == "I":
        payment_complements = cfdi.payments
        if payment_complements:
            response = ""
            for p in payment_complements:
                response += str(p.comprobante.uuid) + '\n' + \
                            '- ' + p.comprobante.name + '\n'
            return response[:-1]


def format_relaciones(cfdi: SatCFDI):
    response = ""
    rel = cfdi.get("CfdiRelacionados")
    if rel:
        response += str(rel["TipoRelacion"]) + '\n'
        for uuid in rel["CfdiRelacionado"]:
            response += str(uuid) + '\n'

    for c in cfdi.relations:
        response += str(c.cfdi_relacionados["TipoRelacion"]) + '\n' + \
                    str(c.comprobante.uuid) + '\n' + \
                    "- " + c.comprobante.name + '\n'
    return response[:-1]


def format_conceptos(cfdi):
    return "\n".join([str(c["ClaveProdServ"]) + "\n- " + c['Descripcion'] for c in cfdi["Conceptos"]])



from decimal import Decimal
from datetime import datetime, date, time
from collections.abc import Sequence
from ...cfdi import CFDI, XElement


class Inmueble(XElement):
    """
    http://www.sat.gob.mx/servicioparcialconstruccion
    Nodo requerido para expresar la información del inmueble en el que se proporcionan los servicios parciales de construcción.
    """
    def __init__(
            self,
            calle: str,
            municipio: str,
            estado: str,
            codigo_postal: str,
            no_exterior: str = None,
            no_interior: str = None,
            colonia: str = None,
            localidad: str = None,
            referencia: str = None,
    ): 
        """
        Nodo requerido para expresar la información del inmueble en el que se proporcionan los servicios parciales de construcción.
        
        :param calle: Este atributo requerido sirve para precisar la avenida, calle, camino o carretera del inmueble
        :param municipio: Atributo requerido que sirve para precisar el municipio o delegación (en el caso del Distrito Federal) en donde se da la ubicación del inmueble.
        :param estado: Entidad Federativa donde se ubica el inmueble conforme al catálogo publicado en el portal del SAT en Internet.
        :param codigo_postal: Atributo requerido que sirve para asentar el código postal en donde se da la ubicación del inmueble.
        :param no_exterior: Este atributo opcional sirve para expresar el número particular en donde se da la ubicación del inmueble en una calle dada.
        :param no_interior: Este atributo opcional sirve para expresar información adicional para especificar la ubicación cuando calle y número exterior (noExterior) no resulten suficientes para determinar la ubicación precisa del inmueble.
        :param colonia: Este atributo opcional sirve para precisar la colonia en donde se da la ubicación del inmueble cuando se desea ser más específico en casos de ubicaciones urbanas.
        :param localidad: Atributo opcional que sirve para precisar la ciudad o población donde se da la ubicación del inmueble.
        :param referencia: Atributo opcional para expresar una referencia adicional de ubicación del inmueble.
        """
        
        super().__init__({
            'Calle': calle,
            'Municipio': municipio,
            'Estado': estado,
            'CodigoPostal': codigo_postal,
            'NoExterior': no_exterior,
            'NoInterior': no_interior,
            'Colonia': colonia,
            'Localidad': localidad,
            'Referencia': referencia,
        })
        

class Parcialesconstruccion(CFDI):
    """
    Complemento para incorporar información de servicios parciales de construcción de inmuebles destinados a casa habitación.
    """
    tag = '{http://www.sat.gob.mx/servicioparcialconstruccion}parcialesconstruccion'
    version = '1.0'
    
    def __init__(
            self,
            num_per_lico_aut: str,
            inmueble: Inmueble | dict,
    ): 
        """
        Complemento para incorporar información de servicios parciales de construcción de inmuebles destinados a casa habitación.
        
        :param num_per_lico_aut: Atributo requerido para expresar el número de permiso, licencia o autorización de construcción proporcionado por el prestatario de los servicios parciales de construcción.
        :param inmueble: Nodo requerido para expresar la información del inmueble en el que se proporcionan los servicios parciales de construcción.
        """
        
        super().__init__({
            'Version': self.version,
            'NumPerLicoAut': num_per_lico_aut,
            'Inmueble': inmueble,
        })
        




from decimal import Decimal
from datetime import datetime, date, time
from collections.abc import Sequence
from ...cfdi import CFDI, XElement


class Extra(XElement):
    """
    http://www.diverza.com/ns/addenda/diverza/1
    Tipo definido para determinar cualquier información útil que se requiera expresar en la versión impresa del Comprobante.
    """
    def __init__(
            self,
            indicador: str = None,
            atributo: str = None,
            valor: str = None,
            prefijo: str = None,
            sufijo: str = None,
    ): 
        """
        Tipo definido para determinar cualquier información útil que se requiera expresar en la versión impresa del Comprobante.
        
        :param indicador: Atributo opcional para señalar un valor que sea útil para identificar el atributo extra
        :param atributo: Atributo opcional para especificar el nombre de un atributo adicional que se desee incluir en la Addenda
        :param valor: Dato opcional que especifica el valor del atributo determinado en el elemento extra
        :param prefijo: Valor opcional que puede incorporarse como prefijo para una mejor identificación del elemento extra
        :param sufijo: Valor opcional que puede incorporarse como sufijo para una mejor identificación del elemento extra
        """
        
        super().__init__({
            'Indicador': indicador,
            'Atributo': atributo,
            'Valor': valor,
            'Prefijo': prefijo,
            'Sufijo': sufijo,
        })
        

class DatosContacto(XElement):
    """
    http://www.diverza.com/ns/addenda/diverza/1
    Tipo de dato para determinar información de datos de contacto como teléfono, email y otros.
    """
    def __init__(
            self,
            telefono: str = None,
            email_comercial: str = None,
            email_contacto: str = None,
            web: str = None,
    ): 
        """
        Tipo de dato para determinar información de datos de contacto como teléfono, email y otros.
        
        :param telefono: Atributo opcional para especificar el teléfono de contacto.
        :param email_comercial: Atributo opcional para especificar el email comerical que se quiera publicar a los clientes.
        :param email_contacto: Atributo opcional para especificar el email de contacto para notificaciones privadas.
        :param web: Atributo opcional para especificar la URL del sitio Web.
        """
        
        super().__init__({
            'Telefono': telefono,
            'EmailComercial': email_comercial,
            'EmailContacto': email_contacto,
            'Web': web,
        })
        

class Ubicacion(XElement):
    """
    http://www.diverza.com/ns/addenda/diverza/1
    Tipo definido para expresar domicilios o direcciones
    """
    def __init__(
            self,
            tax_id: str = None,
            codigo_sitio: str = None,
            calle: str = None,
            numero: str = None,
            colonia: str = None,
            ciudad: str = None,
            municipio: str = None,
            estado: str = None,
            pais: str = None,
            codigo_postal: str = None,
    ): 
        """
        Tipo definido para expresar domicilios o direcciones
        
        :param tax_id: Atributo opcional para colocar la clave de identificación fiscal del receptor.
        :param codigo_sitio: Elemento opcional para especificar el código de sitio del domicilio.
        :param calle: Este atributo opcional sirve para precisar la avenida, calle, camino o carretera donde se da la ubicación.
        :param numero: Este atributo opcional sirve para expresar el número particular en donde se da la ubicación sobre una calle dada.
        :param colonia: Este atributo opcional sirve para precisar la colonia en donde se da la ubicación cuando se desea ser más específico en casos de ubicaciones urbanas.
        :param ciudad: Atributo opcional que sirve para precisar la ciudad o población donde se da la ubicación.
        :param municipio: Atributo opcional que sirve para precisar el municipio o delegación (en el caso del Distrito Federal) en donde se da la ubicación.
        :param estado: Atributo opcional que sirve para precisar el estado o entidad federativa donde se da la ubicación.
        :param pais: Atributo opcional que sirve para precisar el país donde se da la ubicación.
        :param codigo_postal: Atributo opcional que sirve para asentar el código postal en donde se da la ubicación.
        """
        
        super().__init__({
            'TaxID': tax_id,
            'CodigoSitio': codigo_sitio,
            'Calle': calle,
            'Numero': numero,
            'Colonia': colonia,
            'Ciudad': ciudad,
            'Municipio': municipio,
            'Estado': estado,
            'Pais': pais,
            'CodigoPostal': codigo_postal,
        })
        

class Concepto(XElement):
    """
    http://www.diverza.com/ns/addenda/diverza/1
    Nodo de información para especificar información adicional a un concepto del Comprobante
    """
    def __init__(
            self,
            indicador: str = None,
            identificador1: str = None,
            identificador2: str = None,
            descripcion_extranjera: str = None,
            valor_unitario_moneda_extranjera: Decimal | int = None,
            importe_moneda_extranjera: Decimal | int = None,
            mensaje: str = None,
            unidad_medida: str = None,
            codigo_ean: str = None,
            sku: str = None,
            nombre_transportista_c: str = None,
            numero_entrega_c: str = None,
            fecha_entrega_c: date = None,
            datos_extra_c: Extra | dict | Sequence[Extra | dict] = None,
    ): 
        """
        Nodo de información para especificar información adicional a un concepto del Comprobante
        
        :param indicador: Atributo opcional para definir un identificador básico que relacione la información de la Addenda con los nodos Concepto del Comprobante
        :param identificador1: Atributo opcional para especificar un identificador especial útil al Concepto
        :param identificador2: Atributo opcional para especificar otro identificador especial útil al Concepto
        :param descripcion_extranjera: Valor opcional si se requeire agregar información descripriva del Concepto en un idioma diferente al español
        :param valor_unitario_moneda_extranjera: Atributo opcional para determinar el Valor Unitario en alguna moneada extranjera
        :param importe_moneda_extranjera: Atributo opcional para determinar el importe total del concepto en una moneda extranjera
        :param mensaje: Atributo opcional para especificar cualquier mensaje de tipo comercial relacionado al Concepto
        :param unidad_medida: Atributo opcional para determinar alguna unidad de medida adicional que se desee especificar en el Concepto
        :param codigo_ean: Atributo opcional para especificar si se desea el código de barras EAN (European Article Number)
        :param sku: Atributo opcional para especificar el 'Stock-keeping unit' o SKU registrado internamente por el negocio
        :param nombre_transportista_c: Atributo opcional para proporcionar el nombre o clave del servicio de transporte usado para el traslado del atrículo definido en el Concepto
        :param numero_entrega_c: Atributo opcional para determinar el número de registro originado en el momento de entrega del producto relacionado al Concepto facturado
        :param fecha_entrega_c: Atributo opcional para indicar la fecha de entrega del producto relacionado al concepto facturado
        :param datos_extra_c: Elemento para determinar datos adicionales que no están contemplados en el esquema pero que se desean especificar en el Comprobante
        """
        
        super().__init__({
            'Indicador': indicador,
            'Identificador1': identificador1,
            'Identificador2': identificador2,
            'DescripcionExtranjera': descripcion_extranjera,
            'ValorUnitarioMonedaExtranjera': valor_unitario_moneda_extranjera,
            'ImporteMonedaExtranjera': importe_moneda_extranjera,
            'Mensaje': mensaje,
            'UnidadMedida': unidad_medida,
            'CodigoEAN': codigo_ean,
            'Sku': sku,
            'NombreTransportistaC': nombre_transportista_c,
            'NumeroEntregaC': numero_entrega_c,
            'FechaEntregaC': fecha_entrega_c,
            'DatosExtraC': datos_extra_c,
        })
        

class Conceptos(XElement):
    def __init__(
            self,
            concepto: Concepto | dict | Sequence[Concepto | dict],
            numero_conceptos: int = None,
    ): 
        """
        
        :param concepto: Nodo de información para especificar información adicional a un concepto del Comprobante
        :param numero_conceptos: Atributo opcional definido para especificar el número de líneas de concepto presentes en el documento.
        """
        
        super().__init__({
            'Concepto': concepto,
            'NumeroConceptos': numero_conceptos,
        })
        

class Receptor(XElement):
    """
    http://www.diverza.com/ns/addenda/diverza/1
    Nodo opcional para agregar información relacionada al receptor del documento.
    """
    def __init__(
            self,
            comprador: str = None,
            num_cliente: str = None,
            datos_contacto_r: DatosContacto | dict = None,
            domicilio_fiscal_r: Ubicacion | dict = None,
            destino: Ubicacion | dict | Sequence[Ubicacion | dict] = None,
    ): 
        """
        Nodo opcional para agregar información relacionada al receptor del documento.
        
        :param comprador: Atributo opcional para colocar la clave ó código del comprador.
        :param num_cliente: Atributo opcional para colocar la clave o número de cliente asignado al receptor por parte del emisor del documento.
        :param datos_contacto_r: Elemento opcional para agregar datos de contacto del receptor.
        :param domicilio_fiscal_r: Elemento opcional para especificar la información del domicilio del receptor.
        :param destino:
        """
        
        super().__init__({
            'Comprador': comprador,
            'NumCliente': num_cliente,
            'DatosContactoR': datos_contacto_r,
            'DomicilioFiscalR': domicilio_fiscal_r,
            'Destino': destino,
        })
        

class SucursalE(XElement):
    """
    http://www.diverza.com/ns/addenda/diverza/1
    Nodo opcional para agregar información de la sucursal en la que fue expedido el documento
    """
    def __init__(
            self,
            alias: str = None,
            domicilio_sucursal: Ubicacion | dict = None,
            datos_contacto: DatosContacto | dict = None,
    ): 
        """
        Nodo opcional para agregar información de la sucursal en la que fue expedido el documento
        
        :param alias: Atributo opcional para especificar el nombre o clave de la sucursal en la que fue expedido el documento
        :param domicilio_sucursal: Elemento opcional para especificar la información del domicilio de la sucursal en la que fue expedido el documento
        :param datos_contacto: Elemento opcional para especificar datos de contacto de la sucursal en la que fue expedido el documento
        """
        
        super().__init__({
            'Alias': alias,
            'DomicilioSucursal': domicilio_sucursal,
            'DatosContacto': datos_contacto,
        })
        

class Emisor(XElement):
    """
    http://www.diverza.com/ns/addenda/diverza/1
    Nodo opcional para proporcionar información adicional relacionada al emisor.
    """
    def __init__(
            self,
            vendedor: str = None,
            numero_proveedor: str = None,
            tipo_proveedor: str = None,
            gln: str = None,
            datos_contacto_e: DatosContacto | dict = None,
            domicilio_fiscal_e: Ubicacion | dict = None,
            domicilio_origen_e: Ubicacion | dict = None,
            sucursal_e: SucursalE | dict = None,
    ): 
        """
        Nodo opcional para proporcionar información adicional relacionada al emisor.
        
        :param vendedor: Atributo opcional para indicar el nombre o clave del Agente de Ventas relacionado con el documento
        :param numero_proveedor: Atributo opcional para colocar la clave o número de proveedor que el emisor tiene asignado por su cliente.
        :param tipo_proveedor: Atributo opcional para colocar el tipo de proveedor que el emisor tiene asignado por su cliente.
        :param gln: Atributo opcional para colocar el número GLN del proveedor.
        :param datos_contacto_e: Nodo opcional para agregar información de contacto del emisor
        :param domicilio_fiscal_e: Nodo opcional para agregar información del domicilo fiscal del emisor
        :param domicilio_origen_e: Elemento opcional para especificar la información del domicilio de origen de envío de la mercancía.
        :param sucursal_e: Nodo opcional para agregar información de la sucursal en la que fue expedido el documento
        """
        
        super().__init__({
            'Vendedor': vendedor,
            'NumeroProveedor': numero_proveedor,
            'TipoProveedor': tipo_proveedor,
            'Gln': gln,
            'DatosContactoE': datos_contacto_e,
            'DomicilioFiscalE': domicilio_fiscal_e,
            'DomicilioOrigenE': domicilio_origen_e,
            'SucursalE': sucursal_e,
        })
        

class ClavesDescripcion(XElement):
    """
    http://www.diverza.com/ns/addenda/diverza/1
    Nodo opcional para expresar las descripciones de los valores del Comprobante que tienen un catálogo oficial SAT relacionado.
    """
    def __init__(
            self,
            c_forma_pago: str = None,
            c_moneda: str = None,
            c_tipo_de_comprobante: str = None,
            c_metodo_pago: str = None,
            c_lugar_expedicion: str = None,
            c_tipo_relacion: str = None,
            c_regimen_fiscal: str = None,
            c_residencia_fiscal: str = None,
            c_uso_cfdi: str = None,
    ): 
        """
        Nodo opcional para expresar las descripciones de los valores del Comprobante que tienen un catálogo oficial SAT relacionado.
        
        :param c_forma_pago: Atributo opcional para expresar la descripción de la clave del atributo FormaPago del Comprobante conforme al catálogo c_FormaPago.
        :param c_moneda: Atributo opcional para expresar la descripción de la clave del atributo Moneda del Comprobante conforme al catálogo c_Moneda.
        :param c_tipo_de_comprobante: Atributo opcional para expresar la descripción de la clave del atributo TipoDeComprobante del Comprobante conforme al catálogo c_TipoDeComprobante.
        :param c_metodo_pago: Atributo opcional para expresar la descripción de la clave del atributo MetodoPago del Comprobante conforme al catálogo c_MetodoPago.
        :param c_lugar_expedicion: Atributo opcional para expresar la Ciudad y Estado al que corresponda el Código Postal señalado en el atributo LugarExpedicion del Comprobante conforme al catálogo c_CodigoPostal.
        :param c_tipo_relacion: Atributo opcional para expresar la descripción que corresponda a la clave del Tipo de Relación entre CFDIs señalado en el atributo TipoRelacion del nodo CfdiRelacionados del Comprobante conforme al catálogo c_TipoRelacion.
        :param c_regimen_fiscal: Atributo opcional para expresar la descripción que corresponda a la clave del Régimen Fiscal del nodo Emisor del Comprobante conforme al catálogo c_RegimenFiscal.
        :param c_residencia_fiscal: Atributo opcional para expresar la descripción que corresponda a la clave del País del atributo ResidenciaFiscal del nodo Receptor del Comprobante cuando este sea extranjero y conforme al catálogo c_Pais.
        :param c_uso_cfdi: Atributo opcional para expresar la descripción que corresponda a la clave del Uso del CFDI del atributo UsoCFDI del nodo Receptor del Comprobante conforme al catálogo c_UsoCFDI.
        """
        
        super().__init__({
            'CFormaPago': c_forma_pago,
            'CMoneda': c_moneda,
            'CTipoDeComprobante': c_tipo_de_comprobante,
            'CMetodoPago': c_metodo_pago,
            'CLugarExpedicion': c_lugar_expedicion,
            'CTipoRelacion': c_tipo_relacion,
            'CRegimenFiscal': c_regimen_fiscal,
            'CResidenciaFiscal': c_residencia_fiscal,
            'CUsoCFDI': c_uso_cfdi,
        })
        

class Generales(XElement):
    """
    http://www.diverza.com/ns/addenda/diverza/1
    Nodo opcional para proporcionar información general que aplique al documento.
    """
    def __init__(
            self,
            tipo_documento: str = None,
            total_con_letra: str = None,
            observaciones: str = None,
            numero_orden: str = None,
            nombre_transportista: str = None,
            embarque: str = None,
            numero_entrega: str = None,
            terminos_pago: str = None,
            fecha_entrega: date = None,
            fecha_tipo_cambio: date = None,
    ): 
        """
        Nodo opcional para proporcionar información general que aplique al documento.
        
        :param tipo_documento: Atributo opcional para indicar el nombre comercial del CFDI descrito en letra. Por ejemplo: Factura, Nota de Crédito, Recibo de Arrendamiento, etc.
        :param total_con_letra: Atributo opcional para indicar el importe total del CFDI descrito en letra.
        :param observaciones: Atributo opcional para agregar cualquier información de texto que sea útil para el CFDI
        :param numero_orden: Atributo opcional para identificar el número de orden comercial relacionado con el documento
        :param nombre_transportista: Atributo opcional para especificar el nombre o identificador del servico de transporte de la mercancía que ampara el documento..
        :param embarque: Atributo opcional para especificar el dato o código de embarque.
        :param numero_entrega: Atributo opcional para identificar el número de la orden de entrega de la mercancía que ampara el documento.
        :param terminos_pago: Atributo opcional para identificar los terminos de Pago del documento.
        :param fecha_entrega: Atributo opcional para identificar la fecha de entrega de la mercancía que ampara el documento.
        :param fecha_tipo_cambio: Atributo opcional para identificar la fecha del tipo de cambio aplicado en el atributo TipoCambio del nodo Comprobante del CFDI cuando la Moneda corresponda a un valor diferente a Pesos Mexicanos.
        """
        
        super().__init__({
            'TipoDocumento': tipo_documento,
            'TotalConLetra': total_con_letra,
            'Observaciones': observaciones,
            'NumeroOrden': numero_orden,
            'NombreTransportista': nombre_transportista,
            'Embarque': embarque,
            'NumeroEntrega': numero_entrega,
            'TerminosPago': terminos_pago,
            'FechaEntrega': fecha_entrega,
            'FechaTipoCambio': fecha_tipo_cambio,
        })
        

class Diverza(CFDI):
    tag = '{http://www.diverza.com/ns/addenda/diverza/1}diverza'
    version = '1.1'
    
    def __init__(
            self,
            generales: Generales | dict = None,
            claves_descripcion: ClavesDescripcion | dict = None,
            emisor: Emisor | dict = None,
            receptor: Receptor | dict = None,
            conceptos: Conceptos | dict = None,
            complemento: Extra | dict | Sequence[Extra | dict] = None,
    ): 
        """
        
        :param generales: Nodo opcional para proporcionar información general que aplique al documento.
        :param claves_descripcion: Nodo opcional para expresar las descripciones de los valores del Comprobante que tienen un catálogo oficial SAT relacionado.
        :param emisor: Nodo opcional para proporcionar información adicional relacionada al emisor.
        :param receptor: Nodo opcional para agregar información relacionada al receptor del documento.
        :param conceptos:
        :param complemento:
        """
        
        super().__init__({
            'Version': self.version,
            'Generales': generales,
            'ClavesDescripcion': claves_descripcion,
            'Emisor': emisor,
            'Receptor': receptor,
            'Conceptos': conceptos,
            'Complemento': complemento,
        })
        




from decimal import Decimal
from datetime import datetime, date, time
from collections.abc import Sequence
from ...cfdi import CFDI, XElement


class Entidad(XElement):
    """
    http://www.sat.gob.mx/ine
    Nodo condicional para expresar los datos correspondientes a las entidades federativas en las que se va a aplicar el gasto.
    """
    def __init__(
            self,
            clave_entidad: str,
            ambito: str = None,
            contabilidad: int | Sequence[int] = None,
    ): 
        """
        Nodo condicional para expresar los datos correspondientes a las entidades federativas en las que se va a aplicar el gasto.
        
        :param clave_entidad: Atributo requerido para registrar la clave de la entidad a la que aplica el gasto.
        :param ambito: Atributo condicional para registrar el tipo de ámbito de un proceso de tipo Campaña o Precampaña. este atributo no se debe registrar para los procesos de tipo Ordinario.
        :param contabilidad: Nodo para expresar la clave de contabilidad de aspirantes precandidatos, candidatos y concentradoras.
        """
        
        super().__init__({
            'ClaveEntidad': clave_entidad,
            'Ambito': ambito,
            'Contabilidad': contabilidad,
        })
        

class INE(CFDI):
    """
    Complemento al Comprobante Fiscal Digital por Internet (CFDI) para incluir los datos que identifiquen el tipo de proceso al que van dirigidos los gastos que realizan los partidos o las Asociaciones Civiles.
    """
    tag = '{http://www.sat.gob.mx/ine}INE'
    version = '1.0'
    
    def __init__(
            self,
            tipo_proceso: str,
            tipo_comite: str = None,
            id_contabilidad: int = None,
            entidad: Entidad | dict | Sequence[Entidad | dict] = None,
    ): 
        """
        Complemento al Comprobante Fiscal Digital por Internet (CFDI) para incluir los datos que identifiquen el tipo de proceso al que van dirigidos los gastos que realizan los partidos o las Asociaciones Civiles.
        
        :param tipo_proceso: Atributo requerido para expresar el tipo de proceso de que se trate.
        :param tipo_comite: Atributo condicional para expresar el tipo de comité de que se trate.
        :param id_contabilidad: Atributo opcional para registrar la clave de contabilidad de aspirantes precandidatos, candidatos y concentradoras, si se trata de un tipo de proceso ordinario y un comité ejecutivo nacional.Para los otros casos, la clave de contabilidad se registra en el atributo ine:Entidad:Contabilidad:IdContabilidad.
        :param entidad: Nodo condicional para expresar los datos correspondientes a las entidades federativas en las que se va a aplicar el gasto.
        """
        
        super().__init__({
            'Version': self.version,
            'TipoProceso': tipo_proceso,
            'TipoComite': tipo_comite,
            'IdContabilidad': id_contabilidad,
            'Entidad': entidad,
        })
        




from decimal import Decimal
from datetime import datetime, date, time
from collections.abc import Sequence
from ...cfdi import CFDI, XElement


class Domicilio(XElement):
    """
    http://www.sat.gob.mx/CartaPorte
    Nodo opcional para registrar el domicilio de la(s) persona(s) a quién(es) se debe notificar de la llegada del medio de transporte con los bienes o mercancías que se trasladan.
    """
    def __init__(
            self,
            calle: str,
            estado: str,
            pais: str,
            codigo_postal: str,
            numero_exterior: str = None,
            numero_interior: str = None,
            colonia: str = None,
            localidad: str = None,
            referencia: str = None,
            municipio: str = None,
    ): 
        """
        Nodo opcional para registrar el domicilio de la(s) persona(s) a quién(es) se debe notificar de la llegada del medio de transporte con los bienes o mercancías que se trasladan.
        
        :param calle: Atributo requerido que sirve para precisar la calle en que está ubicado el domicilio de la persona notificada del arribo del medio de transporte con los bienes o mercancías que se trasladan.
        :param estado: Atributo requerido para precisar el estado, entidad, región, comunidad, u otra figura análoga en donde se encuentra ubicado el domicilio de la persona notificada del arribo del medio de transporte con los bienes o mercancías que se trasladan.
        :param pais: Atributo requerido que sirve para precisar la clave del país en donde se encuentra ubicado el domicilio de la persona notificada del arribo del medio de transporte con los bienes o mercancías que se trasladan, conforme al catálogo c_Pais publicado en el portal del SAT en Internet que está basado en la especificación ISO 3166-1.
        :param codigo_postal: Atributo requerido para asentar el código postal (PO, BOX) en donde se encuentra ubicado el domicilio de la persona notificada del arribo del medio de transporte con los bienes o mercancías que se trasladan.
        :param numero_exterior: Atributo opcional que sirve para expresar el número exterior en donde se ubica el domicilio de la persona notificada del arribo del medio de transporte con los bienes o mercancías que se trasladan.
        :param numero_interior: Atributo opcional que sirve para expresar el número interior, en caso de existir, en donde se ubica el domicilio de la persona notificada del arribo del medio de transporte con los bienes o mercancías que se trasladan.
        :param colonia: Atributo opcional que sirve para expresar la clave de la colonia o dato análogo en donde se ubica el domicilio de la persona notificada del arribo del medio de transporte con los bienes o mercancías que se trasladan.
        :param localidad: Atributo opcional para precisar la clave de la ciudad, población, distrito u análogo en donde se encuentra ubicado el domicilio de la persona notificada del arribo del medio de transporte con los bienes o mercancías que se trasladan.
        :param referencia: Atributo opcional para expresar una referencia geográfica adicional que permita una más fácil o precisa ubicación de la persona notificada del arribo del medio de transporte con los bienes o mercancías que se trasladan; por ejemplo, las coordenadas GPS.
        :param municipio: Atributo opcional para precisar la clave del municipio, delegación o alcaldía, condado u otro análogo en donde se encuentra ubicado el domicilio de la persona notificada del arribo del medio de transporte con los bienes o mercancías que se trasladan.
        """
        
        super().__init__({
            'Calle': calle,
            'Estado': estado,
            'Pais': pais,
            'CodigoPostal': codigo_postal,
            'NumeroExterior': numero_exterior,
            'NumeroInterior': numero_interior,
            'Colonia': colonia,
            'Localidad': localidad,
            'Referencia': referencia,
            'Municipio': municipio,
        })
        

class Notificado(XElement):
    """
    http://www.sat.gob.mx/CartaPorte
    Nodo opcional para indicar los datos de identificación de la(s) persona(s) a quién(es) se debe notificar de la llegada del medio de transporte con los bienes o mercancías que se trasladan.
    """
    def __init__(
            self,
            rfc_notificado: str = None,
            nombre_notificado: str = None,
            num_reg_id_trib_notificado: str = None,
            residencia_fiscal_notificado: str = None,
            domicilio: Domicilio | dict = None,
    ): 
        """
        Nodo opcional para indicar los datos de identificación de la(s) persona(s) a quién(es) se debe notificar de la llegada del medio de transporte con los bienes o mercancías que se trasladan.
        
        :param rfc_notificado: Atributo condicional para registrar el RFC de la persona a la que se debe notificar del arribo del medio de transporte con los bienes o mercancías que se trasladan.
        :param nombre_notificado: Atributo opcional para registrar el nombre de la persona a la que se debe notificar del arribo del medio de transporte con los bienes o mercancías que se trasladan.
        :param num_reg_id_trib_notificado: Atributo condicional para incorporar el número de identificación o registro fiscal del país de residencia para los efectos fiscales de la persona notificada del arribo del medio de transporte con los bienes o mercancías que se trasladan, cuando sea residente en el extranjero.
        :param residencia_fiscal_notificado: Atributo condicional para registrar la clave del país de residencia para efectos fiscales de la persona notificada del arribo del transporte, conforme con el catálogo c_Pais publicado en el portal del SAT en Internet que está basado en la especificación ISO 3166-1.
        :param domicilio: Nodo opcional para registrar el domicilio de la(s) persona(s) a quién(es) se debe notificar de la llegada del medio de transporte con los bienes o mercancías que se trasladan.
        """
        
        super().__init__({
            'RFCNotificado': rfc_notificado,
            'NombreNotificado': nombre_notificado,
            'NumRegIdTribNotificado': num_reg_id_trib_notificado,
            'ResidenciaFiscalNotificado': residencia_fiscal_notificado,
            'Domicilio': domicilio,
        })
        

class Arrendatario(XElement):
    """
    http://www.sat.gob.mx/CartaPorte
    Nodo condicional para indicar los datos del (los) arrendatario(s) del medio de transporte, siempre que el emisor del comprobante sea distinto al dueño del transporte.
    """
    def __init__(
            self,
            rfc_arrendatario: str = None,
            nombre_arrendatario: str = None,
            num_reg_id_trib_arrendatario: str = None,
            residencia_fiscal_arrendatario: str = None,
            domicilio: Domicilio | dict = None,
    ): 
        """
        Nodo condicional para indicar los datos del (los) arrendatario(s) del medio de transporte, siempre que el emisor del comprobante sea distinto al dueño del transporte.
        
        :param rfc_arrendatario: Atributo condicional para registrar el RFC del arrendatario del medio de transporte.
        :param nombre_arrendatario: Atributo opcional para registrar el nombre del arrendatario del medio de transporte.
        :param num_reg_id_trib_arrendatario: Atributo condicional para incorporar el número de identificación o registro fiscal del país de residencia para los efectos fiscales del arrendatario del medio de transporte, cuando sea residente en el extranjero.
        :param residencia_fiscal_arrendatario: Atributo condicional para registrar la clave del país de residencia para efectos fiscales del arrendatario del transporte, conforme con el catálogo c_Pais publicado en el portal del SAT en Internet que está basado en la especificación ISO 3166-1.
        :param domicilio: Nodo opcional para registrar el domicilio del arrendatario del transporte.
        """
        
        super().__init__({
            'RFCArrendatario': rfc_arrendatario,
            'NombreArrendatario': nombre_arrendatario,
            'NumRegIdTribArrendatario': num_reg_id_trib_arrendatario,
            'ResidenciaFiscalArrendatario': residencia_fiscal_arrendatario,
            'Domicilio': domicilio,
        })
        

class Propietario(XElement):
    """
    http://www.sat.gob.mx/CartaPorte
    Nodo condicional para indicar los datos del (los) propietario(s) del medio de transporte, siempre que el emisor del comprobante sea distinto al dueño del transporte.
    """
    def __init__(
            self,
            rfc_propietario: str = None,
            nombre_propietario: str = None,
            num_reg_id_trib_propietario: str = None,
            residencia_fiscal_propietario: str = None,
            domicilio: Domicilio | dict = None,
    ): 
        """
        Nodo condicional para indicar los datos del (los) propietario(s) del medio de transporte, siempre que el emisor del comprobante sea distinto al dueño del transporte.
        
        :param rfc_propietario: Atributo condicional para registrar el RFC del propietario del medio de transporte.
        :param nombre_propietario: Atributo opcional para registrar el nombre del propietario del medio de transporte.
        :param num_reg_id_trib_propietario: Atributo condicional para incorporar el número de identificación o registro fiscal del país de residencia para los efectos fiscales del propietario del transporte, cuando sea residente en el extranjero.
        :param residencia_fiscal_propietario: Atributo condicional para registrar la clave del país de residencia para efectos fiscales del propietario del transporte, conforme con el catálogo c_Pais publicado en el portal del SAT en Internet que está basado en la especificación ISO 3166-1.
        :param domicilio: Nodo opcional para registrar el domicilio del propietario para el traslado de la mercancía.
        """
        
        super().__init__({
            'RFCPropietario': rfc_propietario,
            'NombrePropietario': nombre_propietario,
            'NumRegIdTribPropietario': num_reg_id_trib_propietario,
            'ResidenciaFiscalPropietario': residencia_fiscal_propietario,
            'Domicilio': domicilio,
        })
        

class Operador(XElement):
    """
    http://www.sat.gob.mx/CartaPorte
    Nodo requerido para indicar los datos del operador del autotransporte de carga federal en el que se trasladan los bienes o mercancías.
    """
    def __init__(
            self,
            rfc_operador: str = None,
            num_licencia: str = None,
            nombre_operador: str = None,
            num_reg_id_trib_operador: str = None,
            residencia_fiscal_operador: str = None,
            domicilio: Domicilio | dict = None,
    ): 
        """
        Nodo requerido para indicar los datos del operador del autotransporte de carga federal en el que se trasladan los bienes o mercancías.
        
        :param rfc_operador: Atributo condicional para registrar el RFC del operador del autotransporte de carga federal en el que se trasladan los bienes o mercancías.
        :param num_licencia: Atributo condicional para expresar el número de folio de la licencia o el permiso otorgado al operador del autotransporte de carga federal en el que se trasladan los bienes o mercancías.
        :param nombre_operador: Atributo opcional para registrar el nombre del operador del autotransporte de carga federal en el que se trasladan los bienes o mercancías.
        :param num_reg_id_trib_operador: Atributo condicional para incorporar el número de identificación o registro fiscal del país de residencia para los efectos fiscales del operador del autotransporte de carga federal en el que se trasladan los bienes o mercancías, cuando sea residente en el extranjero.
        :param residencia_fiscal_operador: Atributo condicional para registrar la clave del país de residencia para efectos fiscales del operador del autotransporte de carga federal en el que se trasladan los bienes o mercancías, conforme con el catálogo c_Pais publicado en el portal del SAT en Internet que está basado en la especificación ISO 3166-1.
        :param domicilio: Nodo opcional para registrar la información del domicilio del operador del autotransporte de carga federal en el que se trasladan los bienes o mercancías.
        """
        
        super().__init__({
            'RFCOperador': rfc_operador,
            'NumLicencia': num_licencia,
            'NombreOperador': nombre_operador,
            'NumRegIdTribOperador': num_reg_id_trib_operador,
            'ResidenciaFiscalOperador': residencia_fiscal_operador,
            'Domicilio': domicilio,
        })
        

class Operadores(XElement):
    """
    http://www.sat.gob.mx/CartaPorte
    Nodo condicional para indicar los datos del(los) operador(es) del autotransporte de carga federal en el que se trasladan los bienes o mercancías.
    """
    def __init__(
            self,
            operador: Operador | dict | Sequence[Operador | dict],
    ): 
        """
        Nodo condicional para indicar los datos del(los) operador(es) del autotransporte de carga federal en el que se trasladan los bienes o mercancías.
        
        :param operador: Nodo requerido para indicar los datos del operador del autotransporte de carga federal en el que se trasladan los bienes o mercancías.
        """
        
        super().__init__({
            'Operador': operador,
        })
        

class FiguraTransporte(XElement):
    """
    http://www.sat.gob.mx/CartaPorte
    Nodo opcional para indicar los datos de la figura del transporte que interviene en el traslado de los bienes o mercancías, cuando el dueño del medio de transporte es diferente del emisor del comprobante con el complemento carta porte.
    """
    def __init__(
            self,
            cve_transporte: str,
            operadores: Operadores | dict | Sequence[Operadores | dict] = None,
            propietario: Propietario | dict | Sequence[Propietario | dict] = None,
            arrendatario: Arrendatario | dict | Sequence[Arrendatario | dict] = None,
            notificado: Notificado | dict | Sequence[Notificado | dict] = None,
    ): 
        """
        Nodo opcional para indicar los datos de la figura del transporte que interviene en el traslado de los bienes o mercancías, cuando el dueño del medio de transporte es diferente del emisor del comprobante con el complemento carta porte.
        
        :param cve_transporte: Atributo requerido para expresar la clave que identifica el medio por el cual se transportan los bienes o mercancías.
        :param operadores: Nodo condicional para indicar los datos del(los) operador(es) del autotransporte de carga federal en el que se trasladan los bienes o mercancías.
        :param propietario: Nodo condicional para indicar los datos del (los) propietario(s) del medio de transporte, siempre que el emisor del comprobante sea distinto al dueño del transporte.
        :param arrendatario: Nodo condicional para indicar los datos del (los) arrendatario(s) del medio de transporte, siempre que el emisor del comprobante sea distinto al dueño del transporte.
        :param notificado: Nodo opcional para indicar los datos de identificación de la(s) persona(s) a quién(es) se debe notificar de la llegada del medio de transporte con los bienes o mercancías que se trasladan.
        """
        
        super().__init__({
            'CveTransporte': cve_transporte,
            'Operadores': operadores,
            'Propietario': propietario,
            'Arrendatario': arrendatario,
            'Notificado': notificado,
        })
        

class DerechosDePaso(XElement):
    """
    http://www.sat.gob.mx/CartaPorte
    Nodo opcional para registrar los tipos de derechos de paso cubiertos por el transportista en las vías férreas de las cuales no es concesionario o asignatario, así como la distancia establecida en kilómetros.
    """
    def __init__(
            self,
            tipo_derecho_de_paso: str,
            kilometraje_pagado: Decimal | int,
    ): 
        """
        Nodo opcional para registrar los tipos de derechos de paso cubiertos por el transportista en las vías férreas de las cuales no es concesionario o asignatario, así como la distancia establecida en kilómetros.
        
        :param tipo_derecho_de_paso: Atributo requerido para registrar el valor de la clave del derecho de paso pagado por el transportista en las vías férreas de las cuales no es concesionario o asignatario.
        :param kilometraje_pagado: Atributo requerido para registrar el total de kilómetros pagados por el transportista en las vías férreas de las cuales no es concesionario o asignatario con el derecho de paso.
        """
        
        super().__init__({
            'TipoDerechoDePaso': tipo_derecho_de_paso,
            'KilometrajePagado': kilometraje_pagado,
        })
        

class Contenedor(XElement):
    """
    http://www.sat.gob.mx/CartaPorte
    Nodo opcional para especificar el tipo de contenedor o vagón en el que se trasladan los bienes o mercancías vía férrea.
    """
    def __init__(
            self,
            tipo_contenedor: str,
            peso_contenedor_vacio: Decimal | int,
            peso_neto_mercancia: Decimal | int,
    ): 
        """
        Nodo opcional para especificar el tipo de contenedor o vagón en el que se trasladan los bienes o mercancías vía férrea.
        
        :param tipo_contenedor: Atributo requerido para registrar la clave con las que se identifica el tipo de contenedor o vagón en el que se traslada los bienes o mercancías.
        :param peso_contenedor_vacio: Atributo requerido para registrar el peso en kilogramos del contenedor vacío en el que se trasladan los bienes o mercancías.
        :param peso_neto_mercancia: Atributo requerido para registrar el peso neto en kilogramos de los bienes o mercancías que trasladan en el contenedor.
        """
        
        super().__init__({
            'TipoContenedor': tipo_contenedor,
            'PesoContenedorVacio': peso_contenedor_vacio,
            'PesoNetoMercancia': peso_neto_mercancia,
        })
        

class Carro(XElement):
    """
    http://www.sat.gob.mx/CartaPorte
    Nodo requerido para registrar la información que permite identificar los carros en los que se trasladan los bienes o mercancías vía férrea.
    """
    def __init__(
            self,
            tipo_carro: str,
            matricula_carro: str,
            guia_carro: str,
            toneladas_netas_carro: Decimal | int,
            contenedor: Contenedor | dict | Sequence[Contenedor | dict] = None,
    ): 
        """
        Nodo requerido para registrar la información que permite identificar los carros en los que se trasladan los bienes o mercancías vía férrea.
        
        :param tipo_carro: Atributo requerido para registrar la clave del tipo de carro utilizado para el traslado de los bienes o mercancías vía férrea.
        :param matricula_carro: Atributo requerido para expresar el número de contenedor, carro de ferrocarril o número económico del vehículo en el que se trasladan los bienes o mercancías vía férrea.
        :param guia_carro: Atributo requerido para expresar el número de guía asignado al contenedor, carro de ferrocarril o vehículo en el que se trasladan los bienes o mercancías vía férrea.
        :param toneladas_netas_carro: Atributo requerido para registrar la cantidad de las toneladas netas contenidas en el contenedor, carro de ferrocarril o vehículo en el que se trasladan los bienes o mercancías vía férrea.
        :param contenedor: Nodo opcional para especificar el tipo de contenedor o vagón en el que se trasladan los bienes o mercancías vía férrea.
        """
        
        super().__init__({
            'TipoCarro': tipo_carro,
            'MatriculaCarro': matricula_carro,
            'GuiaCarro': guia_carro,
            'ToneladasNetasCarro': toneladas_netas_carro,
            'Contenedor': contenedor,
        })
        

class TransporteFerroviario(XElement):
    """
    http://www.sat.gob.mx/CartaPorte
    Nodo condicional para registrar la información que permita la identificación del carro o contenedor en el que se transportan los bienes o mercancías vía férrea.
    """
    def __init__(
            self,
            tipo_de_servicio: str,
            carro: Carro | dict | Sequence[Carro | dict],
            nombre_aseg: str = None,
            num_poliza_seguro: str = None,
            concesionario: str = None,
            derechos_de_paso: DerechosDePaso | dict | Sequence[DerechosDePaso | dict] = None,
    ): 
        """
        Nodo condicional para registrar la información que permita la identificación del carro o contenedor en el que se transportan los bienes o mercancías vía férrea.
        
        :param tipo_de_servicio: Atributo requerido para registrar la clave del tipo de servicio proporcionado para el traslado de los bienes o mercancías vía férrea.
        :param carro: Nodo requerido para registrar la información que permite identificar los carros en los que se trasladan los bienes o mercancías vía férrea.
        :param nombre_aseg: Atributo opcional para expresar el nombre de la aseguradora que cubre los riesgos para el traslado de los bienes o mercancías vía férrea.
        :param num_poliza_seguro: Atributo opcional para registrar el número de póliza asignado por la aseguradora para la protección e indemnización por responsabilidad civil para el traslado de los bienes o mercancías vía férrea.
        :param concesionario: Atributo condicional para registrar la clave del RFC del concesionario o asignatario del transporte, siempre que el traslado de la mercancía se efectúe por un concesionario diferente al emisor del comprobante.
        :param derechos_de_paso: Nodo opcional para registrar los tipos de derechos de paso cubiertos por el transportista en las vías férreas de las cuales no es concesionario o asignatario, así como la distancia establecida en kilómetros.
        """
        
        super().__init__({
            'TipoDeServicio': tipo_de_servicio,
            'Carro': carro,
            'NombreAseg': nombre_aseg,
            'NumPolizaSeguro': num_poliza_seguro,
            'Concesionario': concesionario,
            'DerechosDePaso': derechos_de_paso,
        })
        

class TransporteAereo(XElement):
    """
    http://www.sat.gob.mx/CartaPorte
    Nodo condicional para registrar la información que permita la identificación del transporte aéreo por medio del cual se trasladan los bienes o mercancías.
    """
    def __init__(
            self,
            perm_sct: str,
            num_permiso_sct: str,
            matricula_aeronave: str,
            numero_guia: str,
            codigo_transportista: str,
            nombre_aseg: str = None,
            num_poliza_seguro: str = None,
            lugar_contrato: str = None,
            rfc_transportista: str = None,
            num_reg_id_trib_transpor: str = None,
            residencia_fiscal_transpor: str = None,
            nombre_transportista: str = None,
            rfc_embarcador: str = None,
            num_reg_id_trib_embarc: str = None,
            residencia_fiscal_embarc: str = None,
            nombre_embarcador: str = None,
    ): 
        """
        Nodo condicional para registrar la información que permita la identificación del transporte aéreo por medio del cual se trasladan los bienes o mercancías.
        
        :param perm_sct: Atributo requerido para precisar la clave del tipo de permiso proporcionado por la SCT o la autoridad análoga, para el transporte de bienes o mercancías vía aérea.
        :param num_permiso_sct: Atributo requerido para precisar el número de permiso o algún valor análogo proporcionado por la SCT o la autoridad análoga, para el transporte de bienes o mercancías vía aérea.
        :param matricula_aeronave: Atributo requerido para registrar el número de matrícula de la aeronave que opera en territorio nacional y que se compone de valores alfanuméricos más el carácter especial de guion medio “-“, con una longitud de 10 posiciones.
        :param numero_guia: Atributo requerido para registrar el número de guía aérea con el que se trasladan los bienes o mercancías.
        :param codigo_transportista: Atributo requerido para precisar el valor del código que tiene asignado el transportista y debe contener una clave válida del catálogo “catCartaPorte:c_CodigoTransporteAereo”.
        :param nombre_aseg: Atributo opcional para expresar el nombre de la aseguradora que cubre los riesgos del medio que transporta los bienes o mercancías.
        :param num_poliza_seguro: Atributo opcional para registrar el número de póliza asignado por la aseguradora para la protección e indemnización por responsabilidad civil de la aeronave que transporta los bienes o mercancías.
        :param lugar_contrato: Atributo opcional para registrar el lugar, entidad, región, localidad o análoga, donde se celebró el contrato para el traslado de los bienes o mercancías.
        :param rfc_transportista: Atributo opcional para registrar el RFC del transportista de los bienes o mercancías, en caso de que sea diferente del emisor del CFDI.
        :param num_reg_id_trib_transpor: Atributo condicional para incorporar el número de identificación o registro fiscal del país de residencia para los efectos fiscales del transportista, cuando sea residente en el extranjero.
        :param residencia_fiscal_transpor: Atributo condicional para registrar la clave del país de residencia para efectos fiscales del transportista, conforme con el catálogo c_Pais publicado en el portal del SAT en Internet que está basado en la especificación ISO 3166-1.
        :param nombre_transportista: Atributo opcional para registrar el nombre del transportista ya sea nacional o extranjero.
        :param rfc_embarcador: Atributo opcional para registrar el RFC del embarcador de los bienes o mercancías que se trasladan y debe estar en la lista de contribuyentes inscritos no cancelados del SAT (l_RFC).
        :param num_reg_id_trib_embarc: Atributo condicional para incorporar el número de identificación o registro fiscal del país de residencia para los efectos fiscales del embarcador de los bienes o mercancías que se trasladan, cuando sea residente en el extranjero.
        :param residencia_fiscal_embarc: Atributo condicional para registrar la clave del país de residencia para efectos fiscales del embarcador de los bienes o mercancías, conforme con el catálogo c_Pais publicado en el portal del SAT en Internet que está basado en la especificación ISO 3166-1.
        :param nombre_embarcador: Atributo opcional para registrar el nombre del embarcador de los bienes o mercancías que se trasladan, ya sea nacional o extranjero.
        """
        
        super().__init__({
            'PermSCT': perm_sct,
            'NumPermisoSCT': num_permiso_sct,
            'MatriculaAeronave': matricula_aeronave,
            'NumeroGuia': numero_guia,
            'CodigoTransportista': codigo_transportista,
            'NombreAseg': nombre_aseg,
            'NumPolizaSeguro': num_poliza_seguro,
            'LugarContrato': lugar_contrato,
            'RFCTransportista': rfc_transportista,
            'NumRegIdTribTranspor': num_reg_id_trib_transpor,
            'ResidenciaFiscalTranspor': residencia_fiscal_transpor,
            'NombreTransportista': nombre_transportista,
            'RFCEmbarcador': rfc_embarcador,
            'NumRegIdTribEmbarc': num_reg_id_trib_embarc,
            'ResidenciaFiscalEmbarc': residencia_fiscal_embarc,
            'NombreEmbarcador': nombre_embarcador,
        })
        

class TransporteMaritimo(XElement):
    """
    http://www.sat.gob.mx/CartaPorte
    Nodo condicional para registrar la información que permita la identificación de la embarcación por medio del cual se transportan los bienes o mercancías, vía marítima.
    """
    def __init__(
            self,
            tipo_embarcacion: str,
            matricula: str,
            numero_omi: str,
            nacionalidad_embarc: str,
            unidades_de_arq_bruto: Decimal | int,
            tipo_carga: str,
            num_cert_itc: str,
            nombre_agente_naviero: str,
            num_autorizacion_naviero: str,
            contenedor: Contenedor | dict | Sequence[Contenedor | dict],
            perm_sct: str = None,
            num_permiso_sct: str = None,
            nombre_aseg: str = None,
            num_poliza_seguro: str = None,
            anio_embarcacion: int = None,
            nombre_embarc: str = None,
            eslora: Decimal | int = None,
            manga: Decimal | int = None,
            calado: Decimal | int = None,
            linea_naviera: str = None,
            num_viaje: str = None,
            num_conoc_embarc: str = None,
    ): 
        """
        Nodo condicional para registrar la información que permita la identificación de la embarcación por medio del cual se transportan los bienes o mercancías, vía marítima.
        
        :param tipo_embarcacion: Atributo requerido para registrar la clave de identificación del transporte del tipo de embarcación que es utilizado para trasladar los bienes o mercancías.
        :param matricula: Atributo requerido para registrar el número de la matrícula o registro de la embarcación que es utilizada para transportar los bienes o mercancías.
        :param numero_omi: Atributo requerido para registrar el número de identificación asignado por la Organización Marítima Internacional a la embarcación encargada de transportar los bienes o mercancías.
        :param nacionalidad_embarc: Atributo requerido para expresar el país correspondiente a la nacionalidad de la embarcación que transporta los bienes o mercancías.
        :param unidades_de_arq_bruto: Atributo requerido para registrar el valor de las unidades de arqueo bruto conforme a las medidas internacionales definidas por el ITC para cada tipo de buque o embarcación en la que se transportan los bienes o mercancías.
        :param tipo_carga: Atributo requerido para especificar el tipo de carga bajo el cual se tipifican los bienes o mercancías que se transportan en la embarcación.
        :param num_cert_itc: Atributo requerido para registrar el número del certificado emitido por la ITC para la embarcación o buque que transporta los bienes o mercancías.
        :param nombre_agente_naviero: Atributo requerido para registrar el nombre del agente naviero autorizado para gestionar el traslado de los bienes o mercancías vía marítima.
        :param num_autorizacion_naviero: Atributo requerido para expresar el número de registro de autorización como agente naviero consignatario emitido por la SCT.
        :param contenedor: Nodo requerido para registrar los datos del contenedor en el que transportan los bienes o mercancías.
        :param perm_sct: Atributo opcional para precisar la clave del tipo de permiso proporcionado por la SCT, el cual debe corresponder a la navegación que se está utilizando para el traslado de los bienes o mercancías registrado en el catálogo catCartaPorte:c_TipoPermiso.
        :param num_permiso_sct: Atributo opcional para precisar el número de permiso proporcionado por la SCT para la navegación.
        :param nombre_aseg: Atributo opcional para expresar el nombre de la aseguradora que cubre el seguro de protección e indemnización por responsabilidad civil de la embarcación.
        :param num_poliza_seguro: Atributo opcional para registrar el número de póliza asignado por la aseguradora para la protección e indemnización por responsabilidad civil de la embarcación.
        :param anio_embarcacion: Atributo opcional para expresar el año de la embarcación en la que se transportan los bienes o mercancías.
        :param nombre_embarc: Atributo opcional para expresar el nombre de la embarcación en la que se transportan los bienes o mercancías.
        :param eslora: Atributo opcional para registrar la longitud de eslora, definida en pies, con la que cuenta la embarcación o buque en el que se transportan los bienes o mercancías.
        :param manga: Atributo opcional para registrar la longitud de manga, definida en pies, con la que cuenta la embarcación o buque en el que se transportan los bienes o mercancías.
        :param calado: Atributo opcional para registrar la longitud del calado, definida en pies, con la que cuenta la embarcación o buque en el que se transportan los bienes o mercancías.
        :param linea_naviera: Atributo opcional para expresar el nombre de la línea naviera autorizada de gestionar el traslado de los bienes o mercancías vía marítima.
        :param num_viaje: Atributo opcional para expresar el número del viaje con el que se identifica el traslado de los bienes o mercancías en el buque o embarcación.
        :param num_conoc_embarc: Atributo opcional para expresar el número de conocimiento de embarque que identifica el traslado de los bienes o mercancías.
        """
        
        super().__init__({
            'TipoEmbarcacion': tipo_embarcacion,
            'Matricula': matricula,
            'NumeroOMI': numero_omi,
            'NacionalidadEmbarc': nacionalidad_embarc,
            'UnidadesDeArqBruto': unidades_de_arq_bruto,
            'TipoCarga': tipo_carga,
            'NumCertITC': num_cert_itc,
            'NombreAgenteNaviero': nombre_agente_naviero,
            'NumAutorizacionNaviero': num_autorizacion_naviero,
            'Contenedor': contenedor,
            'PermSCT': perm_sct,
            'NumPermisoSCT': num_permiso_sct,
            'NombreAseg': nombre_aseg,
            'NumPolizaSeguro': num_poliza_seguro,
            'AnioEmbarcacion': anio_embarcacion,
            'NombreEmbarc': nombre_embarc,
            'Eslora': eslora,
            'Manga': manga,
            'Calado': calado,
            'LineaNaviera': linea_naviera,
            'NumViaje': num_viaje,
            'NumConocEmbarc': num_conoc_embarc,
        })
        

class Remolque(XElement):
    """
    http://www.sat.gob.mx/CartaPorte
    Nodo requerido para expresar la información del remolque o semirremolques que se emplean con el autotransporte para el traslado de los bienes o mercancías.
    """
    def __init__(
            self,
            sub_tipo_rem: str,
            placa: str,
    ): 
        """
        Nodo requerido para expresar la información del remolque o semirremolques que se emplean con el autotransporte para el traslado de los bienes o mercancías.
        
        :param sub_tipo_rem: Atributo requerido para expresar la clave del subtipo de remolque o semirremolques que se emplean con el autotransporte para el traslado de los bienes o mercancías.
        :param placa: Atributo requerido para registrar el valor de la placa vehicular del remolque o semirremolque que es utilizado para transportar los bienes o mercancías, se deben registrar solo los caracteres alfanuméricos, sin guiones y espacios.
        """
        
        super().__init__({
            'SubTipoRem': sub_tipo_rem,
            'Placa': placa,
        })
        

class IdentificacionVehicular(XElement):
    """
    http://www.sat.gob.mx/CartaPorte
    Nodo requerido para registrar los datos de identificación del autotransporte en el que se trasladan los bienes o mercancías.
    """
    def __init__(
            self,
            config_vehicular: str,
            placa_vm: str,
            anio_modelo_vm: int,
    ): 
        """
        Nodo requerido para registrar los datos de identificación del autotransporte en el que se trasladan los bienes o mercancías.
        
        :param config_vehicular: Atributo requerido para expresar la clave de nomenclatura del autotransporte que es utilizado para transportar los bienes o mercancías.
        :param placa_vm: Atributo requerido para registrar el valor de la placa vehicular del autotransporte que es utilizado para transportar los bienes o mercancías, se deben registrar solo los caracteres alfanuméricos, sin guiones y espacios.
        :param anio_modelo_vm: Atributo requerido para registrar el año del autotransporte que es utilizado para transportar los bienes o mercancías.
        """
        
        super().__init__({
            'ConfigVehicular': config_vehicular,
            'PlacaVM': placa_vm,
            'AnioModeloVM': anio_modelo_vm,
        })
        

class AutotransporteFederal(XElement):
    """
    http://www.sat.gob.mx/CartaPorte
    Nodo condicional para registrar la información que permita la identificación del autotransporte de carga federal, por medio del cual se transportan los bienes o mercancías, que transitan a través de las carreteras federales del territorio nacional.
    """
    def __init__(
            self,
            perm_sct: str,
            num_permiso_sct: str,
            nombre_aseg: str,
            num_poliza_seguro: str,
            identificacion_vehicular: IdentificacionVehicular | dict,
            remolques: Remolque | dict | Sequence[Remolque | dict] = None,
    ): 
        """
        Nodo condicional para registrar la información que permita la identificación del autotransporte de carga federal, por medio del cual se transportan los bienes o mercancías, que transitan a través de las carreteras federales del territorio nacional.
        
        :param perm_sct: Atributo requerido para precisar la clave del tipo de permiso proporcionado por la SCT, el cual debe corresponder de acuerdo al tipo de autotransporte utilizado para el traslado de los bienes o mercancías registrado en el catálogo catCartaPorte:c_TipoPermiso.
        :param num_permiso_sct: Atributo requerido para precisar el número del permiso otorgado por la SCT, el cual se debe capturar de acuerdo al tipo de autotransporte utilizado para el traslado de los bienes o mercancías.
        :param nombre_aseg: Atributo requerido para expresar el nombre de la aseguradora que cubre los riesgos del autotransporte utilizado para el traslado de los bienes o mercancías.
        :param num_poliza_seguro: Atributo requerido para registrar el número de póliza asignado por la aseguradora, que cubre los riesgos del autotransporte utilizado para el traslado de los bienes o mercancías.
        :param identificacion_vehicular: Nodo requerido para registrar los datos de identificación del autotransporte en el que se trasladan los bienes o mercancías.
        :param remolques: Nodo opcional para registrar los datos del (los) remolque(s) o semirremolque(s) que se emplean con el autotransporte para el traslado de los bienes o mercancías.
        """
        
        super().__init__({
            'PermSCT': perm_sct,
            'NumPermisoSCT': num_permiso_sct,
            'NombreAseg': nombre_aseg,
            'NumPolizaSeguro': num_poliza_seguro,
            'IdentificacionVehicular': identificacion_vehicular,
            'Remolques': remolques,
        })
        

class DetalleMercancia(XElement):
    """
    http://www.sat.gob.mx/CartaPorte
    Nodo condicional para registrar mayor detalle de los bienes o mercancías que se transportan y será requerido cuando el traslado sea vía marítima.
    """
    def __init__(
            self,
            unidad_peso: str,
            peso_bruto: Decimal | int,
            peso_neto: Decimal | int,
            peso_tara: Decimal | int,
            num_piezas: int = None,
    ): 
        """
        Nodo condicional para registrar mayor detalle de los bienes o mercancías que se transportan y será requerido cuando el traslado sea vía marítima.
        
        :param unidad_peso: Atributo requerido para expresar la clave de unidad de medida estandarizada del peso de los bienes o mercancías que se trasladan.
        :param peso_bruto: Atributo requerido para expresar el peso total bruto de los bienes o mercancías que se trasladan.
        :param peso_neto: Atributo requerido para expresar el peso total neto de los bienes o mercancías que se trasladan.
        :param peso_tara: Atributo requerido para expresar el peso bruto, menos el peso neto de las mercancías que se trasladan.
        :param num_piezas: Atributo opcional para registrar el número de piezas de los bienes o mercancías que se trasladan.
        """
        
        super().__init__({
            'UnidadPeso': unidad_peso,
            'PesoBruto': peso_bruto,
            'PesoNeto': peso_neto,
            'PesoTara': peso_tara,
            'NumPiezas': num_piezas,
        })
        

class CantidadTransporta(XElement):
    """
    http://www.sat.gob.mx/CartaPorte
    Nodo condicional para registrar la cantidad de los bienes o mercancías que se trasladan en los distintos medios de transporte, que será distribuida o captada en distintos puntos, a fin de identificar el punto de origen y destino de dichos bienes o mercancías, y se podrá registrar este nodo tantas veces sea necesario.
    """
    def __init__(
            self,
            cantidad: Decimal | int,
            id_origen: str,
            id_destino: str,
            cves_transporte: str = None,
    ): 
        """
        Nodo condicional para registrar la cantidad de los bienes o mercancías que se trasladan en los distintos medios de transporte, que será distribuida o captada en distintos puntos, a fin de identificar el punto de origen y destino de dichos bienes o mercancías, y se podrá registrar este nodo tantas veces sea necesario.
        
        :param cantidad: Atributo requerido para expresar el número de bienes o mercancías que se trasladan en los distintos medios de transporte.
        :param id_origen: Atributo requerido para expresar la clave del identificador del origen de los bienes o mercancías que se trasladan por los distintos medios de transporte, de acuerdo al valor registrado en el nodo “Origen”, del elemento “Ubicacion”.
        :param id_destino: Atributo requerido para expresar la clave del identificador del destino de los bienes o mercancías que se trasladan por los distintos medios de transporte, de acuerdo al valor registrado en el nodo “Origen”, del elemento “Ubicacion”.
        :param cves_transporte: Atributo condicional para expresar la clave que identifica el medio por el cual se transportan los bienes o mercancías, dicha clave debe ser distinta a “05” que corresponde a “Ducto”.
        """
        
        super().__init__({
            'Cantidad': cantidad,
            'IDOrigen': id_origen,
            'IDDestino': id_destino,
            'CvesTransporte': cves_transporte,
        })
        

class Mercancia(XElement):
    """
    http://www.sat.gob.mx/CartaPorte
    Nodo requerido para registrar información de los bienes o mercancías que se trasladan en los distintos medios de transporte.
    """
    def __init__(
            self,
            peso_en_kg: Decimal | int,
            bienes_transp: str = None,
            clave_stcc: str = None,
            descripcion: str = None,
            cantidad: Decimal | int = None,
            clave_unidad: str = None,
            unidad: str = None,
            dimensiones: str = None,
            material_peligroso: str = None,
            cve_material_peligroso: str = None,
            embalaje: str = None,
            descrip_embalaje: str = None,
            valor_mercancia: Decimal | int = None,
            moneda: str = None,
            fraccion_arancelaria: str = None,
            uuid_comercio_ext: str = None,
            cantidad_transporta: CantidadTransporta | dict | Sequence[CantidadTransporta | dict] = None,
            detalle_mercancia: DetalleMercancia | dict = None,
    ): 
        """
        Nodo requerido para registrar información de los bienes o mercancías que se trasladan en los distintos medios de transporte.
        
        :param peso_en_kg: Atributo requerido para indicar el peso en kilogramos de los bienes o mercancías que se trasladan en los distintos medios de transporte.
        :param bienes_transp: Atributo condicional para expresar la clave de producto de los bienes o mercancías que se trasladan en los distintos medios de transporte.
        :param clave_stcc: Atributo opcional para expresar la clave de producto conforme al catálogo de la STCC cuando el medio de transporte es ferroviario.
        :param descripcion: Atributo condicional para describir los bienes o mercancías que se trasladan en los distintos medios de transporte.
        :param cantidad: Atributo condicional para expresar la cantidad de los bienes o mercancías que se trasladan en los distintos medios de transporte.
        :param clave_unidad: Atributo condicional para precisar la clave de unidad de medida estandarizada aplicable para la cantidad de los bienes o mercancías que se trasladan en los distintos medios de transporte. La unidad debe corresponder con la descripción de los bienes o mercancías registrados.
        :param unidad: Atributo opcional para precisar la unidad de medida propia de los bienes o mercancías que se trasladan en los distintos medios de transporte, aplicable para la cantidad. La unidad debe corresponder con la descripción de los bienes o mercancías.
        :param dimensiones: Atributo opcional para expresar las medidas del empaque de los bienes o mercancías que se trasladan vía aérea. Se debe registrar la longitud, la altura y la anchura en centímetros o en pulgadas separados dichos valores por una diagonal, i.e. 30/40/30cm
        :param material_peligroso: Atributo condicional para precisar si los bienes o mercancías que se trasladan son considerados material peligroso.
        :param cve_material_peligroso: Atributo condicional para precisar la clave del tipo de material peligroso que se transporta.
        :param embalaje: Atributo condicional para precisar la clave del tipo de embalaje que se requiere para transportar el material o residuo peligroso.
        :param descrip_embalaje: Atributo opcional para expresar la descripción del embalaje en el que se transporta los bienes o mercancías que se consideran material o residuo peligroso.
        :param valor_mercancia: Atributo condicional para expresar el monto del valor de los bienes o mercancías que se trasladan en los distintos medios de transporte, de acuerdo al valor mercado, el valor pactado en la contraprestación o al valor estimado que determine el contribuyente.
        :param moneda: Atributo condicional para identificar la clave de la moneda utilizada para expresar el valor de los bienes o mercancías que se trasladan en los distintos medios de transporte, cuando se usa moneda nacional se registra MXN, conforme con la especificación ISO 4217.
        :param fraccion_arancelaria: Atributo condicional que sirve para expresar la clave de la fracción arancelaria correspondiente a la descripción de los bienes o mercancías que se trasladan en los distintos medios de transporte como importación o exportación, este dato se vuelve requerido cuando el atributo “EntradaSalidaMerc” contenga información. Debe ser conforme con el catálogo c_FraccionArancelaria publicado en el portal del SAT en Internet.
        :param uuid_comercio_ext: Atributo condicional para expresar el folio fiscal o UUID del comprobante de comercio exterior que se relaciona, este dato se vuelve requerido cuando el atributo “EntradaSalidaMerc” contenga el valor “Salida”.
        :param cantidad_transporta: Nodo condicional para registrar la cantidad de los bienes o mercancías que se trasladan en los distintos medios de transporte, que será distribuida o captada en distintos puntos, a fin de identificar el punto de origen y destino de dichos bienes o mercancías, y se podrá registrar este nodo tantas veces sea necesario.
        :param detalle_mercancia: Nodo condicional para registrar mayor detalle de los bienes o mercancías que se transportan y será requerido cuando el traslado sea vía marítima.
        """
        
        super().__init__({
            'PesoEnKg': peso_en_kg,
            'BienesTransp': bienes_transp,
            'ClaveSTCC': clave_stcc,
            'Descripcion': descripcion,
            'Cantidad': cantidad,
            'ClaveUnidad': clave_unidad,
            'Unidad': unidad,
            'Dimensiones': dimensiones,
            'MaterialPeligroso': material_peligroso,
            'CveMaterialPeligroso': cve_material_peligroso,
            'Embalaje': embalaje,
            'DescripEmbalaje': descrip_embalaje,
            'ValorMercancia': valor_mercancia,
            'Moneda': moneda,
            'FraccionArancelaria': fraccion_arancelaria,
            'UUIDComercioExt': uuid_comercio_ext,
            'CantidadTransporta': cantidad_transporta,
            'DetalleMercancia': detalle_mercancia,
        })
        

class Mercancias(XElement):
    """
    http://www.sat.gob.mx/CartaPorte
    Nodo requerido para registrar la información de los bienes o mercancías que se trasladan en los distintos medios de transporte.
    """
    def __init__(
            self,
            num_total_mercancias: int,
            mercancia: Mercancia | dict | Sequence[Mercancia | dict],
            peso_bruto_total: Decimal | int = None,
            unidad_peso: str = None,
            peso_neto_total: Decimal | int = None,
            cargo_por_tasacion: Decimal | int = None,
            autotransporte_federal: AutotransporteFederal | dict = None,
            transporte_maritimo: TransporteMaritimo | dict = None,
            transporte_aereo: TransporteAereo | dict = None,
            transporte_ferroviario: TransporteFerroviario | dict = None,
    ): 
        """
        Nodo requerido para registrar la información de los bienes o mercancías que se trasladan en los distintos medios de transporte.
        
        :param num_total_mercancias: Atributo requerido para expresar el número total de los bienes o mercancías que se trasladan en los distintos medios de transporte, identificándose por cada nodo "Mercancia" registrado en el complemento.
        :param mercancia: Nodo requerido para registrar información de los bienes o mercancías que se trasladan en los distintos medios de transporte.
        :param peso_bruto_total: Atributo condicional para expresar la suma del peso bruto total de los bienes o mercancías que se trasladan por transporte ferroviario y aéreo.
        :param unidad_peso: Atributo condicional para expresar la clave de unidad de medida estandarizada del peso de los bienes o mercancías que se trasladan vía férrea y aérea.
        :param peso_neto_total: Atributo condicional para registrar la suma de los valores registrados en el atributo “PesoNeto” del nodo “DetalleMercancia” del elemento “Mercancia”.
        :param cargo_por_tasacion: Atributo opcional para expresar el importe pagado por la tasación de los bienes o mercancías que se trasladan vía aérea.
        :param autotransporte_federal: Nodo condicional para registrar la información que permita la identificación del autotransporte de carga federal, por medio del cual se transportan los bienes o mercancías, que transitan a través de las carreteras federales del territorio nacional.
        :param transporte_maritimo: Nodo condicional para registrar la información que permita la identificación de la embarcación por medio del cual se transportan los bienes o mercancías, vía marítima.
        :param transporte_aereo: Nodo condicional para registrar la información que permita la identificación del transporte aéreo por medio del cual se trasladan los bienes o mercancías.
        :param transporte_ferroviario: Nodo condicional para registrar la información que permita la identificación del carro o contenedor en el que se transportan los bienes o mercancías vía férrea.
        """
        
        super().__init__({
            'NumTotalMercancias': num_total_mercancias,
            'Mercancia': mercancia,
            'PesoBrutoTotal': peso_bruto_total,
            'UnidadPeso': unidad_peso,
            'PesoNetoTotal': peso_neto_total,
            'CargoPorTasacion': cargo_por_tasacion,
            'AutotransporteFederal': autotransporte_federal,
            'TransporteMaritimo': transporte_maritimo,
            'TransporteAereo': transporte_aereo,
            'TransporteFerroviario': transporte_ferroviario,
        })
        

class Destino(XElement):
    """
    http://www.sat.gob.mx/CartaPorte
    Nodo condicional para registrar la información detallada del Destino de los bienes o mercancías que se trasladan.
    """
    def __init__(
            self,
            fecha_hora_prog_llegada: datetime,
            id_destino: str = None,
            rfc_destinatario: str = None,
            nombre_destinatario: str = None,
            num_reg_id_trib: str = None,
            residencia_fiscal: str = None,
            num_estacion: str = None,
            nombre_estacion: str = None,
            navegacion_trafico: str = None,
    ): 
        """
        Nodo condicional para registrar la información detallada del Destino de los bienes o mercancías que se trasladan.
        
        :param fecha_hora_prog_llegada: Atributo requerido para la expresión de la fecha y hora en la que estima arriben a su destino los bienes o mercancías. Se expresa en la forma AAAA-MM-DDThh:mm:ss.
        :param id_destino: Atributo condicional para registrar una clave que sirva para identificar el punto de llegada de los bienes o mercancías que se trasladan por los distintos medios de transporte, mediante un folio, el cual estará compuesto de la siguiente forma: el acrónimo “DE” seguido de 6 dígitos numéricos asignados por el contribuyente que emite el comprobante para su identificación.
        :param rfc_destinatario: Atributo condicional para registrar el RFC del destinatario al que se le entregarán los bienes o mercancías que se trasladan.
        :param nombre_destinatario: Atributo opcional para registrar el nombre del destinatario de los bienes o mercancías que se trasladan.
        :param num_reg_id_trib: Atributo condicional para incorporar el número de identificación o registro fiscal del país de residencia para los efectos fiscales del destinatario de los bienes o mercancías que se trasladan, cuando sea residente en el extranjero.
        :param residencia_fiscal: Atributo condicional para registrar la clave del país de residencia para efectos fiscales del destinatario de los bienes o mercancías, conforme con el catálogo c_Pais publicado en el portal del SAT en Internet que está basado en la especificación ISO 3166-1.
        :param num_estacion: Atributo condicional para registrar la clave del número de la estación de llegada por la que se trasladan los bienes o mercancías en los distintos medios de transporte, esto de acuerdo al valor de la columna Clave identificación que permite asociarla al tipo de transporte.
        :param nombre_estacion: Atributo condicional para registrar el nombre de la estación por la que se trasladan los bienes o mercancías en los distintos medios de transporte.
        :param navegacion_trafico: Atributo condicional para registrar el tipo de puerto por el que se documentan los bienes o mercancías vía marítima.
        """
        
        super().__init__({
            'FechaHoraProgLlegada': fecha_hora_prog_llegada,
            'IDDestino': id_destino,
            'RFCDestinatario': rfc_destinatario,
            'NombreDestinatario': nombre_destinatario,
            'NumRegIdTrib': num_reg_id_trib,
            'ResidenciaFiscal': residencia_fiscal,
            'NumEstacion': num_estacion,
            'NombreEstacion': nombre_estacion,
            'NavegacionTrafico': navegacion_trafico,
        })
        

class Origen(XElement):
    """
    http://www.sat.gob.mx/CartaPorte
    Nodo condicional para registrar la información detallada del Origen de los bienes o mercancías que se trasladan.
    """
    def __init__(
            self,
            fecha_hora_salida: datetime,
            id_origen: str = None,
            rfc_remitente: str = None,
            nombre_remitente: str = None,
            num_reg_id_trib: str = None,
            residencia_fiscal: str = None,
            num_estacion: str = None,
            nombre_estacion: str = None,
            navegacion_trafico: str = None,
    ): 
        """
        Nodo condicional para registrar la información detallada del Origen de los bienes o mercancías que se trasladan.
        
        :param fecha_hora_salida: Atributo requerido para la expresión de la fecha y hora estimada en la que salen los bienes o mercancías del origen. Se expresa en la forma AAAA-MM-DDThh:mm:ss.
        :param id_origen: Atributo condicional para registrar una clave que sirva para identificar el punto de salida de los bienes o mercancías que se trasladan por los distintos medios de transporte, mediante un folio, el cual estará compuesto de la siguiente forma: el acrónimo “OR” seguido de 6 dígitos numéricos asignados por el contribuyente que emite el comprobante para su identificación.
        :param rfc_remitente: Atributo condicional para registrar el RFC del remitente de los bienes o mercancías que se trasladan.
        :param nombre_remitente: Atributo opcional para registrar el nombre del remitente de los bienes o mercancías que se trasladan.
        :param num_reg_id_trib: Atributo condicional para incorporar el número de identificación o registro fiscal del país de residencia para los efectos fiscales del remitente de los bienes o mercancías que se trasladan, cuando sea residente en el extranjero.
        :param residencia_fiscal: Atributo condicional para registrar la clave del país de residencia para efectos fiscales del remitente de los bienes o mercancías, conforme con el catálogo c_Pais publicado en el portal del SAT en Internet que está basado en la especificación ISO 3166-1.
        :param num_estacion: Atributo condicional para registrar la clave del número de la estación de salida por la que se trasladan los bienes o mercancías en los distintos medios de transporte, esto de acuerdo al valor de la columna Clave identificación que permite asociarla al tipo de transporte.
        :param nombre_estacion: Atributo condicional para registrar el nombre de la estación por la que se trasladan los bienes o mercancías en los distintos medios de transporte.
        :param navegacion_trafico: Atributo condicional para registrar el tipo de puerto por el que se documentan los bienes o mercancías vía marítima.
        """
        
        super().__init__({
            'FechaHoraSalida': fecha_hora_salida,
            'IDOrigen': id_origen,
            'RFCRemitente': rfc_remitente,
            'NombreRemitente': nombre_remitente,
            'NumRegIdTrib': num_reg_id_trib,
            'ResidenciaFiscal': residencia_fiscal,
            'NumEstacion': num_estacion,
            'NombreEstacion': nombre_estacion,
            'NavegacionTrafico': navegacion_trafico,
        })
        

class Ubicacion(XElement):
    """
    http://www.sat.gob.mx/CartaPorte
    Nodo requerido para registrar la ubicación que sirve para reflejar el domicilio del origen y/o destino parcial o final que tienen los bienes o mercancías que se trasladan por distintos medios de transporte.
    """
    def __init__(
            self,
            tipo_estacion: str = None,
            distancia_recorrida: Decimal | int = None,
            origen: Origen | dict = None,
            destino: Destino | dict = None,
            domicilio: Domicilio | dict = None,
    ): 
        """
        Nodo requerido para registrar la ubicación que sirve para reflejar el domicilio del origen y/o destino parcial o final que tienen los bienes o mercancías que se trasladan por distintos medios de transporte.
        
        :param tipo_estacion: Atributo condicional para precisar el tipo de estación por el que pasan los bienes o mercancías durante su traslado en los distintos medios de transporte.
        :param distancia_recorrida: Atributo condicional para registrar la distancia recorrida en kilómetros de la ubicación de Origen a la de Destino parcial o final, de los distintos medios de transporte que trasladan los bienes o mercancías.
        :param origen: Nodo condicional para registrar la información detallada del Origen de los bienes o mercancías que se trasladan.
        :param destino: Nodo condicional para registrar la información detallada del Destino de los bienes o mercancías que se trasladan.
        :param domicilio: Nodo condicional para registrar información del domicilio de origen y/o destino de los bienes o mercancías que se trasladan en los distintos medios de transporte.
        """
        
        super().__init__({
            'TipoEstacion': tipo_estacion,
            'DistanciaRecorrida': distancia_recorrida,
            'Origen': origen,
            'Destino': destino,
            'Domicilio': domicilio,
        })
        

class CartaPorte(CFDI):
    """
    Complemento para incorporar al Comprobante Fiscal Digital por Internet (CFDI), la información relacionada a los bienes o mercancías, ubicaciones de origen, puntos intermedios y destino, así como lo referente al medio por el que se transportan; ya sea por vía terrestre (carretera federal o líneas férreas), aérea, marítima o fluvial; además de incluir el traslado de Hidrocarburos y Petrolíferos.
    """
    tag = '{http://www.sat.gob.mx/CartaPorte}CartaPorte'
    version = '1.0'
    
    def __init__(
            self,
            transp_internac: str,
            ubicaciones: Ubicacion | dict | Sequence[Ubicacion | dict],
            mercancias: Mercancias | dict,
            entrada_salida_merc: str = None,
            via_entrada_salida: str = None,
            total_dist_rec: Decimal | int = None,
            figura_transporte: FiguraTransporte | dict = None,
    ): 
        """
        Complemento para incorporar al Comprobante Fiscal Digital por Internet (CFDI), la información relacionada a los bienes o mercancías, ubicaciones de origen, puntos intermedios y destino, así como lo referente al medio por el que se transportan; ya sea por vía terrestre (carretera federal o líneas férreas), aérea, marítima o fluvial; además de incluir el traslado de Hidrocarburos y Petrolíferos.
        
        :param transp_internac: Atributo requerido para expresar si los bienes o mercancías que son transportadas ingresan o salen del territorio nacional.
        :param ubicaciones: Nodo requerido para registrar las distintas ubicaciones que sirven para reflejar el domicilio del origen y/o destino que tienen los bienes o mercancías que se trasladan por distintos medios de transporte.
        :param mercancias: Nodo requerido para registrar la información de los bienes o mercancías que se trasladan en los distintos medios de transporte.
        :param entrada_salida_merc: Atributo condicional para precisar si los bienes o mercancías ingresan o salen del territorio nacional.
        :param via_entrada_salida: Atributo condicional para precisar la vía de ingreso o salida de los bienes o mercancías en territorio nacional.
        :param total_dist_rec: Atributo condicional para registrar la suma de las distancias recorridas en kilómetros, registradas en el atributo “Ubicaciones:Ubicacion:DistanciaRecorrida” para el traslado de los bienes o mercancías.
        :param figura_transporte: Nodo opcional para indicar los datos de la figura del transporte que interviene en el traslado de los bienes o mercancías, cuando el dueño del medio de transporte es diferente del emisor del comprobante con el complemento carta porte.
        """
        
        super().__init__({
            'Version': self.version,
            'TranspInternac': transp_internac,
            'Ubicaciones': ubicaciones,
            'Mercancias': mercancias,
            'EntradaSalidaMerc': entrada_salida_merc,
            'ViaEntradaSalida': via_entrada_salida,
            'TotalDistRec': total_dist_rec,
            'FiguraTransporte': figura_transporte,
        })
        




from decimal import Decimal
from datetime import datetime, date, time
from collections.abc import Sequence
from ...cfdi import CFDI, XElement


class Concepto(XElement):
    """
    http://www.sat.gob.mx/valesdedespensa
    Nodo requerido para la expresión de una transacción a ser reportada por el proveedor del monedero electrónico de vales de despensa.
    """
    def __init__(
            self,
            identificador: str,
            fecha: datetime,
            rfc: str,
            curp: str,
            nombre: str,
            importe: Decimal | int,
            num_seguridad_social: str = None,
    ): 
        """
        Nodo requerido para la expresión de una transacción a ser reportada por el proveedor del monedero electrónico de vales de despensa.
        
        :param identificador: Atributo requerido para expresar el identificador o numero del monedero electrónico.
        :param fecha: Atributo requerido para la expresión de la Fecha y hora de expedición de la operación reportada. Se expresa en la forma aaaa-mm-ddThh:mm:ss, de acuerdo con la especificación ISO 8601.
        :param rfc: Atributo requerido para la expresión del Registro Federal de Contribuyentes del trabajador al que se le otorgó el monedero electrónico sin guiones o espacios
        :param curp: Atributo requerido para la expresión de la CURP del trabajador al que se le otorgó el monedero electrónico.
        :param nombre: Atributo requerido para la expresión del Nombre del trabajador al que se le otorgó el monedero electrónico sin guiones o espacios
        :param importe: Atributo requerido para expresar el importe del depósito efectuado al trabajador en el monedero electrónico.
        :param num_seguridad_social: Atributo opcional para la expresión del numero de seguridad social aplicable al trabajador.
        """
        
        super().__init__({
            'Identificador': identificador,
            'Fecha': fecha,
            'Rfc': rfc,
            'Curp': curp,
            'Nombre': nombre,
            'Importe': importe,
            'NumSeguridadSocial': num_seguridad_social,
        })
        

class ValesDeDespensa(CFDI):
    """
    Complemento al Comprobante Fiscal Digital por Internet (CFDI) para integrar la información emitida por un prestador de servicios de monedero electrónico de vales de despensa.
    """
    tag = '{http://www.sat.gob.mx/valesdedespensa}ValesDeDespensa'
    version = '1.0'
    tipo_operacion = 'monedero electrónico'
    
    def __init__(
            self,
            numero_de_cuenta: str,
            total: Decimal | int,
            conceptos: Concepto | dict | Sequence[Concepto | dict],
            registro_patronal: str = None,
    ): 
        """
        Complemento al Comprobante Fiscal Digital por Internet (CFDI) para integrar la información emitida por un prestador de servicios de monedero electrónico de vales de despensa.
        
        :param numero_de_cuenta: Atributo requerido para expresar el numero de cuenta del adquiriente del monedero electrónico.
        :param total: Atributo requerido para expresar el monto total de vales de despensa otorgados.
        :param conceptos: Nodo requerido para enlistar los conceptos cubiertos por los monederos electrónicos de vales de despensa.
        :param registro_patronal: Atributo opcional para expresar el registro patronal del adquirente del monedero electrónico.
        """
        
        super().__init__({
            'Version': self.version,
            'TipoOperacion': self.tipo_operacion,
            'NumeroDeCuenta': numero_de_cuenta,
            'Total': total,
            'Conceptos': conceptos,
            'RegistroPatronal': registro_patronal,
        })
        




import re

CURP_Regex = "[A-Z][AEIOUX][A-Z]{2}[0-9]{2}(0[1-9]|1[012])(0[1-9]|[12][0-9]|3[01])[MH]([ABCMTZ]S|[BCJMOT]C|[CNPST]L|[GNQ]T|[GQS]R|C[MH]|[MY]N|[DH]G|NE|VZ|DF|SP)[BCDFGHJ-NP-TV-Z]{3}[0-9A-Z][0-9]"
CURP_Verify_Chars = "0123456789ABCDEFGHIJKLMNÑOPQRSTUVWXYZ"


class CURP(str):
    def __new__(cls, value):
        value = value.upper()
        match = re.fullmatch(CURP_Regex, value)
        if match:
            self = super().__new__(cls, value)
            self.estado = match.group(3)
            return self
        else:
            raise ValueError("CURP Not Valid")

    def is_valid(self):
        tot = 0

        for i, c in enumerate(self[::-1], start=1):
            tot += CURP_Verify_Chars.index(c) * i

        return tot % 10 == 0

    def __repr__(self):
        return '%s(%s)' % (
            self.__class__.__qualname__,
            f'{super().__repr__()}'
        )



from decimal import Decimal
from datetime import datetime, date, time
from collections.abc import Sequence
from ...cfdi import CFDI, XElement


class RetEfectFideicomiso(XElement):
    """
    http://www.sat.gob.mx/esquemas/retencionpago/1/fideicomisonoempresarial
    Nodo requerido para expresar las retenciones efectuadas al fideicomiso
    """
    def __init__(
            self,
            mont_ret_rel_pag_fideic: Decimal | int,
            desc_ret_rel_pag_fideic: str,
    ): 
        """
        Nodo requerido para expresar las retenciones efectuadas al fideicomiso
        
        :param mont_ret_rel_pag_fideic: Atributo requerido para expresar el monto de la retenciones con relación al fideicomiso
        :param desc_ret_rel_pag_fideic: Atributo requerido para expresar la descripción de las retenciones con relación al fideicomiso
        """
        
        super().__init__({
            'MontRetRelPagFideic': mont_ret_rel_pag_fideic,
            'DescRetRelPagFideic': desc_ret_rel_pag_fideic,
        })
        

class DeduccOSalidas(XElement):
    """
    http://www.sat.gob.mx/esquemas/retencionpago/1/fideicomisonoempresarial
    Nodo requerido para expresar el importe de los egresos del periodo de fideicomisos que no realizan actividades empresariales
    """
    def __init__(
            self,
            mont_tot_egres_periodo: Decimal | int,
            part_prop_del_fideicom: Decimal | int,
            prop_del_mont_tot: Decimal | int,
            integrac_egresos: str,
    ): 
        """
        Nodo requerido para expresar el importe de los egresos del periodo de fideicomisos que no realizan actividades empresariales
        
        :param mont_tot_egres_periodo: Atributo requerido para expresar el importe total de los egresos del periodo de fideicomiso que no realizan actividades empresariales
        :param part_prop_del_fideicom: Atributo requerido para expresar la parte proporcional de las deducciones autorizadas del periodo que corresponden al fideicomisario o fideicomitente
        :param prop_del_mont_tot: Atributo requerido para expresar la proporción de participación del fideicomisario o fideicomitente de acuerdo al contrato
        :param integrac_egresos: Nodo requerido para expresar la integración de las deducciones en los ingresos obtenidos por fideicomisos que no realizan actividades empresariales
        """
        
        super().__init__({
            'MontTotEgresPeriodo': mont_tot_egres_periodo,
            'PartPropDelFideicom': part_prop_del_fideicom,
            'PropDelMontTot': prop_del_mont_tot,
            'IntegracEgresos': integrac_egresos,
        })
        

class IngresosOEntradas(XElement):
    """
    http://www.sat.gob.mx/esquemas/retencionpago/1/fideicomisonoempresarial
    Nodo requerido para expresar el importe de los ingresos del periodo de fideicomisos que no realizan actividades empresariales
    """
    def __init__(
            self,
            mont_tot_entradas_periodo: Decimal | int,
            part_prop_acum_del_fideicom: Decimal | int,
            prop_del_mont_tot: Decimal | int,
            integrac_ingresos: str,
    ): 
        """
        Nodo requerido para expresar el importe de los ingresos del periodo de fideicomisos que no realizan actividades empresariales
        
        :param mont_tot_entradas_periodo: Atributo requerido para expresar el importe total de los ingresos del periodo de los fideicomisos que no realizan actividades empresariales
        :param part_prop_acum_del_fideicom: Atributo requerido para expresar la parte proporcional de los ingresos acumulables del periodo que correspondan al fideicomisario o fideicomitente
        :param prop_del_mont_tot: Atributo requerido para expresar la proporción de participación del fideicomisario o fideicomitente de acuerdo al contrato
        :param integrac_ingresos: Nodo requerido para expresar el detalle de la integración de los ingresos obtenidos en el periodo por fideicomisos que no realizan actividades empresariales
        """
        
        super().__init__({
            'MontTotEntradasPeriodo': mont_tot_entradas_periodo,
            'PartPropAcumDelFideicom': part_prop_acum_del_fideicom,
            'PropDelMontTot': prop_del_mont_tot,
            'IntegracIngresos': integrac_ingresos,
        })
        

class Fideicomisonoempresarial(CFDI):
    """
    Complemento para expresar la información sobre los fideicomisos que no realizan actividades empresariales
    """
    tag = '{http://www.sat.gob.mx/esquemas/retencionpago/1/fideicomisonoempresarial}Fideicomisonoempresarial'
    version = '1.0'
    
    def __init__(
            self,
            ingresos_oentradas: IngresosOEntradas | dict,
            deducc_osalidas: DeduccOSalidas | dict,
            ret_efect_fideicomiso: RetEfectFideicomiso | dict,
    ): 
        """
        Complemento para expresar la información sobre los fideicomisos que no realizan actividades empresariales
        
        :param ingresos_oentradas: Nodo requerido para expresar el importe de los ingresos del periodo de fideicomisos que no realizan actividades empresariales
        :param deducc_osalidas: Nodo requerido para expresar el importe de los egresos del periodo de fideicomisos que no realizan actividades empresariales
        :param ret_efect_fideicomiso: Nodo requerido para expresar las retenciones efectuadas al fideicomiso
        """
        
        super().__init__({
            'Version': self.version,
            'IngresosOEntradas': ingresos_oentradas,
            'DeduccOSalidas': deducc_osalidas,
            'RetEfectFideicomiso': ret_efect_fideicomiso,
        })
        




from ..models import Signer, Certificate


class Issuer:
    """
    Instancia generica de Emisor para usar en la creacion y sellado de comprobantes
    """

    def __init__(self, tax_system: str, signer: Signer | Certificate = None, legal_name: str = None, rfc: str = None, certificate_number: str = None):
        """
        Issuer to issue and stamp CFDIs

        :param signer: Llave y Certificado del Emisor
        :param legal_name: Legal Name (Razon Social)
        :param tax_system: Atributo requerido para incorporar la clave del régimen del contribuyente emisor al que aplicará el efecto fiscal de este comprobante.
        """
        self.signer = signer if isinstance(signer, Signer) else None
        self.certificate = signer
        self.rfc = rfc or signer.rfc
        self.legal_name = legal_name or signer.legal_name
        self.tax_system = tax_system
        self.certificate_number = certificate_number or (signer.certificate_number if signer else "")



from decimal import Decimal
from datetime import datetime, date, time
from collections.abc import Sequence
from ...cfdi import CFDI, XElement


class Traslado(XElement):
    """
    http://www.sat.gob.mx/Pagos
    Nodo requerido para la información detallada de un traslado de impuesto específico.
    """
    def __init__(
            self,
            impuesto: str,
            tipo_factor: str,
            tasa_o_cuota: Decimal | int,
            importe: Decimal | int,
    ): 
        """
        Nodo requerido para la información detallada de un traslado de impuesto específico.
        
        :param impuesto: Atributo requerido para señalar la clave del tipo de impuesto trasladado.
        :param tipo_factor: Atributo requerido para señalar la clave del tipo de factor que se aplica a la base del impuesto.
        :param tasa_o_cuota: Atributo requerido para señalar el valor de la tasa o cuota del impuesto que se traslada.
        :param importe: Atributo requerido para señalar el importe del impuesto trasladado. No se permiten valores negativos.
        """
        
        super().__init__({
            'Impuesto': impuesto,
            'TipoFactor': tipo_factor,
            'TasaOCuota': tasa_o_cuota,
            'Importe': importe,
        })
        

class Retencion(XElement):
    """
    http://www.sat.gob.mx/Pagos
    Nodo requerido para registrar la información detallada de una retención de impuesto específico.
    """
    def __init__(
            self,
            impuesto: str,
            importe: Decimal | int,
    ): 
        """
        Nodo requerido para registrar la información detallada de una retención de impuesto específico.
        
        :param impuesto: Atributo requerido para señalar la clave del tipo de impuesto retenido.
        :param importe: Atributo requerido para señalar el importe o monto del impuesto retenido. No se permiten valores negativos.
        """
        
        super().__init__({
            'Impuesto': impuesto,
            'Importe': importe,
        })
        

class Impuestos(XElement):
    """
    http://www.sat.gob.mx/Pagos
    Nodo condicional para expresar el resumen de los impuestos aplicables cuando este documento sea un anticipo.
    """
    def __init__(
            self,
            total_impuestos_retenidos: Decimal | int = None,
            total_impuestos_trasladados: Decimal | int = None,
            retenciones: Retencion | dict | Sequence[Retencion | dict] = None,
            traslados: Traslado | dict | Sequence[Traslado | dict] = None,
    ): 
        """
        Nodo condicional para expresar el resumen de los impuestos aplicables cuando este documento sea un anticipo.
        
        :param total_impuestos_retenidos: Atributo condicional para expresar el total de los impuestos retenidos que se desprenden del pago. No se permiten valores negativos.
        :param total_impuestos_trasladados: Atributo condicional para expresar el total de los impuestos trasladados que se desprenden del pago. No se permiten valores negativos.
        :param retenciones: Nodo condicional para capturar los impuestos retenidos aplicables.
        :param traslados: Nodo condicional para capturar los impuestos trasladados aplicables.
        """
        
        super().__init__({
            'TotalImpuestosRetenidos': total_impuestos_retenidos,
            'TotalImpuestosTrasladados': total_impuestos_trasladados,
            'Retenciones': retenciones,
            'Traslados': traslados,
        })
        

class DoctoRelacionado(XElement):
    """
    http://www.sat.gob.mx/Pagos
    Nodo condicional para expresar la lista de documentos relacionados con los pagos diferentes de anticipos. Por cada documento que se relacione se debe generar un nodo DoctoRelacionado.
    """
    def __init__(
            self,
            id_documento: str,
            moneda_dr: str,
            metodo_de_pago_dr: str,
            serie: str = None,
            folio: str = None,
            tipo_cambio_dr: Decimal | int = None,
            num_parcialidad: int = None,
            imp_saldo_ant: Decimal | int = None,
            imp_pagado: Decimal | int = None,
            imp_saldo_insoluto: Decimal | int = None,
    ): 
        """
        Nodo condicional para expresar la lista de documentos relacionados con los pagos diferentes de anticipos. Por cada documento que se relacione se debe generar un nodo DoctoRelacionado.
        
        :param id_documento: Atributo requerido para expresar el identificador del documento relacionado con el pago. Este dato puede ser un Folio Fiscal de la Factura Electrónica o bien el número de operación de un documento digital.
        :param moneda_dr: Atributo requerido para identificar la clave de la moneda utilizada en los importes del documento relacionado, cuando se usa moneda nacional o el documento relacionado no especifica la moneda se registra MXN. Los importes registrados en los atributos “ImpSaldoAnt”, “ImpPagado” e “ImpSaldoInsoluto” de éste nodo, deben corresponder a esta moneda. Conforme con la especificación ISO 4217.
        :param metodo_de_pago_dr: Atributo requerido para expresar la clave del método de pago que se registró en el documento relacionado.
        :param serie: Atributo opcional para precisar la serie del comprobante para control interno del contribuyente, acepta una cadena de caracteres.
        :param folio: Atributo opcional para precisar el folio del comprobante para control interno del contribuyente, acepta una cadena de caracteres.
        :param tipo_cambio_dr: Atributo condicional para expresar el tipo de cambio conforme con la moneda registrada en el documento relacionado. Es requerido cuando la moneda del documento relacionado es distinta de la moneda de pago. Se debe registrar el número de unidades de la moneda señalada en el documento relacionado que equivalen a una unidad de la moneda del pago. Por ejemplo: El documento relacionado se registra en USD El pago se realiza por 100 EUR. Este atributo se registra como 1.114700 USD/EUR. El importe pagado equivale a 100 EUR * 1.114700 USD/EUR = 111.47 USD.
        :param num_parcialidad: Atributo condicional para expresar el número de parcialidad que corresponde al pago. Es requerido cuando MetodoDePagoDR contiene: “PPD” Pago en parcialidades o diferido.
        :param imp_saldo_ant: Atributo condicional para expresar el monto del saldo insoluto de la parcialidad anterior. Es requerido cuando MetodoDePagoDR contiene: “PPD” Pago en parcialidades o diferido.En el caso de que sea la primer parcialidad este campo debe contener el importe total del documento relacionado.
        :param imp_pagado: Atributo condicional para expresar el importe pagado para el documento relacionado. Es obligatorio cuando exista más de un documento relacionado o cuando existe un documento relacionado y el TipoCambioDR tiene un valor.
        :param imp_saldo_insoluto: Atributo condicional para expresar la diferencia entre el importe del saldo anterior y el monto del pago. Es requerido cuando MetodoDePagoDR contiene: “PPD” Pago en parcialidades o diferido.
        """
        
        super().__init__({
            'IdDocumento': id_documento,
            'MonedaDR': moneda_dr,
            'MetodoDePagoDR': metodo_de_pago_dr,
            'Serie': serie,
            'Folio': folio,
            'TipoCambioDR': tipo_cambio_dr,
            'NumParcialidad': num_parcialidad,
            'ImpSaldoAnt': imp_saldo_ant,
            'ImpPagado': imp_pagado,
            'ImpSaldoInsoluto': imp_saldo_insoluto,
        })
        

class Pago(XElement):
    """
    http://www.sat.gob.mx/Pagos
    Elemento requerido para incorporar la información de la recepción de pagos.
    """
    def __init__(
            self,
            fecha_pago: datetime,
            forma_de_pago_p: str,
            moneda_p: str,
            monto: Decimal | int,
            tipo_cambio_p: Decimal | int = None,
            num_operacion: str = None,
            rfc_emisor_cta_ord: str = None,
            nom_banco_ord_ext: str = None,
            cta_ordenante: str = None,
            rfc_emisor_cta_ben: str = None,
            cta_beneficiario: str = None,
            tipo_cad_pago: str = None,
            cert_pago: str = None,
            cad_pago: str = None,
            sello_pago: str = None,
            docto_relacionado: DoctoRelacionado | dict | Sequence[DoctoRelacionado | dict] = None,
            impuestos: Impuestos | dict | Sequence[Impuestos | dict] = None,
    ): 
        """
        Elemento requerido para incorporar la información de la recepción de pagos.
        
        :param fecha_pago: Atributo requerido para expresar la fecha y hora en la que el beneficiario recibe el pago. Se expresa en la forma aaaa-mm-ddThh:mm:ss, de acuerdo con la especificación ISO 8601.En caso de no contar con la hora se debe registrar 12:00:00.
        :param forma_de_pago_p: Atributo requerido para expresar la clave de la forma en que se realiza el pago.
        :param moneda_p: Atributo requerido para identificar la clave de la moneda utilizada para realizar el pago, cuando se usa moneda nacional se registra MXN. El atributo Pagos:Pago:Monto y los atributos TotalImpuestosRetenidos, TotalImpuestosTrasladados, Traslados:Traslado:Importe y Retenciones:Retencion:Importe del nodo Pago:Impuestos deben ser expresados en esta moneda. Conforme con la especificación ISO 4217.
        :param monto: Atributo requerido para expresar el importe del pago.
        :param tipo_cambio_p: Atributo condicional para expresar el tipo de cambio de la moneda a la fecha en que se realizó el pago. El valor debe reflejar el número de pesos mexicanos que equivalen a una unidad de la divisa señalada en el atributo MonedaP. Es requerido cuando el atributo MonedaP es diferente a MXN.
        :param num_operacion: Atributo condicional para expresar el número de cheque, número de autorización, número de referencia, clave de rastreo en caso de ser SPEI, línea de captura o algún número de referencia análogo que identifique la operación que ampara el pago efectuado
        :param rfc_emisor_cta_ord: Atributo condicional para expresar la clave RFC de la entidad emisora de la cuenta origen, es decir, la operadora, el banco, la institución financiera, emisor de monedero electrónico, etc., en caso de ser extranjero colocar XEXX010101000, considerar las reglas de obligatoriedad publicadas en la página del SAT para éste atributo de acuerdo con el catálogo catCFDI:c_FormaPago.
        :param nom_banco_ord_ext: Atributo condicional para expresar el nombre del banco ordenante, es requerido en caso de ser extranjero. Considerar las reglas de obligatoriedad publicadas en la página del SAT para éste atributo de acuerdo con el catálogo catCFDI:c_FormaPago.
        :param cta_ordenante: Atributo condicional para incorporar el número de la cuenta con la que se realizó el pago. Considerar las reglas de obligatoriedad publicadas en la página del SAT para éste atributo de acuerdo con el catálogo catCFDI:c_FormaPago
        :param rfc_emisor_cta_ben: Atributo condicional para expresar la clave RFC de la entidad operadora de la cuenta destino, es decir, la operadora, el banco, la institución financiera, emisor de monedero electrónico, etc. Considerar las reglas de obligatoriedad publicadas en la página del SAT para éste atributo de acuerdo con el catálogo catCFDI:c_FormaPago.
        :param cta_beneficiario: Atributo condicional para incorporar el número de cuenta en donde se recibió el pago. Considerar las reglas de obligatoriedad publicadas en la página del SAT para éste atributo de acuerdo con el catálogo catCFDI:c_FormaPago.
        :param tipo_cad_pago: Atributo condicional para identificar la clave del tipo de cadena de pago que genera la entidad receptora del pago. Considerar las reglas de obligatoriedad publicadas en la página del SAT para éste atributo de acuerdo con el catálogo catCFDI:c_FormaPago.
        :param cert_pago: Atributo condicional que sirve para incorporar el certificado que ampara al pago, como una cadena de texto en formato base 64. Es requerido en caso de que el atributo TipoCadPago contenga información.
        :param cad_pago: Atributo condicional para expresar la cadena original del comprobante de pago generado por la entidad emisora de la cuenta beneficiaria. Es requerido en caso de que el atributo TipoCadPago contenga información.
        :param sello_pago: Atributo condicional para integrar el sello digital que se asocie al pago. La entidad que emite el comprobante de pago, ingresa una cadena original y el sello digital en una sección de dicho comprobante, este sello digital es el que se debe registrar en este campo. Debe ser expresado como una cadena de texto en formato base 64. Es requerido en caso de que el atributo TipoCadPago contenga información.
        :param docto_relacionado: Nodo condicional para expresar la lista de documentos relacionados con los pagos diferentes de anticipos. Por cada documento que se relacione se debe generar un nodo DoctoRelacionado.
        :param impuestos: Nodo condicional para expresar el resumen de los impuestos aplicables cuando este documento sea un anticipo.
        """
        
        super().__init__({
            'FechaPago': fecha_pago,
            'FormaDePagoP': forma_de_pago_p,
            'MonedaP': moneda_p,
            'Monto': monto,
            'TipoCambioP': tipo_cambio_p,
            'NumOperacion': num_operacion,
            'RfcEmisorCtaOrd': rfc_emisor_cta_ord,
            'NomBancoOrdExt': nom_banco_ord_ext,
            'CtaOrdenante': cta_ordenante,
            'RfcEmisorCtaBen': rfc_emisor_cta_ben,
            'CtaBeneficiario': cta_beneficiario,
            'TipoCadPago': tipo_cad_pago,
            'CertPago': cert_pago,
            'CadPago': cad_pago,
            'SelloPago': sello_pago,
            'DoctoRelacionado': docto_relacionado,
            'Impuestos': impuestos,
        })
        

class Pagos(CFDI):
    """
    Complemento para el Comprobante Fiscal Digital por Internet (CFDI) para registrar información sobre la recepción de pagos. El emisor de este complemento para recepción de pagos debe ser quien las leyes le obligue a expedir comprobantes por los actos o actividades que realicen, por los ingresos que se perciban o por las retenciones de contribuciones que efectúen.
    """
    tag = '{http://www.sat.gob.mx/Pagos}Pagos'
    version = '1.0'
    
    def __init__(
            self,
            pago: Pago | dict | Sequence[Pago | dict],
    ): 
        """
        Complemento para el Comprobante Fiscal Digital por Internet (CFDI) para registrar información sobre la recepción de pagos. El emisor de este complemento para recepción de pagos debe ser quien las leyes le obligue a expedir comprobantes por los actos o actividades que realicen, por los ingresos que se perciban o por las retenciones de contribuciones que efectúen.
        
        :param pago: Elemento requerido para incorporar la información de la recepción de pagos.
        """
        
        super().__init__({
            'Version': self.version,
            'Pago': pago,
        })
        




from decimal import Decimal
from datetime import datetime, date, time
from collections.abc import Sequence
from ...cfdi import CFDI, XElement


class ComisionDelServicio(XElement):
    """
    http://www.sat.gob.mx/esquemas/retencionpago/1/PlataformasTecnologicas10
    Nodo condicional para detallar la información de la comisión pagada por el uso de plataformas tecnológicas por cada servicio prestado o enajenación relacionado.
    """
    def __init__(
            self,
            importe: Decimal | int,
            base: Decimal | int = None,
            porcentaje: Decimal | int = None,
    ): 
        """
        Nodo condicional para detallar la información de la comisión pagada por el uso de plataformas tecnológicas por cada servicio prestado o enajenación relacionado.
        
        :param importe: Atributo requerido para detallar el valor del importe cobrado por la comisión del uso del servicio de las plataformas tecnológicas.
        :param base: Atributo opcional para registrar la base de la comisión del servicio de la plataforma, pagadas por personas físicas o personas morales utilizando plataformas tecnológicas.
        :param porcentaje: Atributo opcional para detallar el valor del porcentaje cobrado por la comisión del uso del servicio de las plataformas tecnológicas.
        """
        
        super().__init__({
            'Importe': importe,
            'Base': base,
            'Porcentaje': porcentaje,
        })
        

class ContribucionGubernamental(XElement):
    """
    http://www.sat.gob.mx/esquemas/retencionpago/1/PlataformasTecnologicas10
    Nodo opcional para detallar la información de las contribuciones gubernamentales pagadas por los servicios realizados por personas físicas utilizando plataformas tecnológicas; por ejemplo, impuesto sobre hospedaje.
    """
    def __init__(
            self,
            imp_contrib: Decimal | int,
            entidad_donde_paga_la_contribucion: str,
    ): 
        """
        Nodo opcional para detallar la información de las contribuciones gubernamentales pagadas por los servicios realizados por personas físicas utilizando plataformas tecnológicas; por ejemplo, impuesto sobre hospedaje.
        
        :param imp_contrib: Atributo requerido para registrar el importe de la contribución gubernamental pagada por los servicios realizados por personas físicas utilizando plataformas tecnológicas.
        :param entidad_donde_paga_la_contribucion: Atributo requerido para registrar la clave de la Entidad Federativa donde se efectúa el pago de la contribución gubernamental.
        """
        
        super().__init__({
            'ImpContrib': imp_contrib,
            'EntidadDondePagaLaContribucion': entidad_donde_paga_la_contribucion,
        })
        

class ImpuestosTrasladadosdelServicio(XElement):
    """
    http://www.sat.gob.mx/esquemas/retencionpago/1/PlataformasTecnologicas10
    Nodo condicional para detallar la información de los impuestos trasladados respecto de las operaciones realizadas por personas físicas o personas morales utilizando plataformas tecnológicas.
    """
    tipo_factor = 'Tasa'
    def __init__(
            self,
            base: Decimal | int,
            impuesto: str,
            tasa_cuota: Decimal | int,
            importe: Decimal | int,
    ): 
        """
        Nodo condicional para detallar la información de los impuestos trasladados respecto de las operaciones realizadas por personas físicas o personas morales utilizando plataformas tecnológicas.
        
        :param base: Atributo requerido para señalar la base para el cálculo del impuesto, la determinación de la base se realiza de acuerdo con las disposiciones fiscales vigentes. No se permiten valores negativos.
        :param impuesto: Atributo requerido para señalar la clave del tipo de impuesto trasladado aplicable al bien o servicio.
        :param tasa_cuota: Atributo requerido para señalar el valor de la tasa o cuota del impuesto que se traslada para el bien o servicio.
        :param importe: Atributo requerido para señalar el importe del impuesto trasladado que aplica al bien o servicio. No se permiten valores negativos.
        """
        
        super().__init__({
            'Base': base,
            'Impuesto': impuesto,
            'TipoFactor': self.tipo_factor,
            'TasaCuota': tasa_cuota,
            'Importe': importe,
        })
        

class DetallesDelServicio(XElement):
    """
    http://www.sat.gob.mx/esquemas/retencionpago/1/PlataformasTecnologicas10
    Nodo requerido para detallar la información de la enajenación de bienes y los tipos de servicios realizadas por personas físicas o personas morales utilizando plataformas tecnológicas.
    """
    def __init__(
            self,
            forma_pago_serv: str,
            tipo_de_serv: str,
            fecha_serv: date,
            precio_serv_sin_iva: Decimal | int,
            sub_tip_serv: str = None,
            rfc_tercero_autorizado: str = None,
            impuestos_trasladadosdel_servicio: ImpuestosTrasladadosdelServicio | dict = None,
            contribucion_gubernamental: ContribucionGubernamental | dict = None,
            comision_del_servicio: ComisionDelServicio | dict = None,
    ): 
        """
        Nodo requerido para detallar la información de la enajenación de bienes y los tipos de servicios realizadas por personas físicas o personas morales utilizando plataformas tecnológicas.
        
        :param forma_pago_serv: Atributo requerido para expresar la clave de la forma de pago con la que se liquida el servicio prestado o la enajenación de bienes.
        :param tipo_de_serv: Atributo requerido para expresar la clave del tipo de servicio prestado o la enajenación de bienes, según corresponda.
        :param fecha_serv: Atributo requerido para expresar la fecha en la que el cliente paga el servicio o el bien.
        :param precio_serv_sin_iva: Atributo requerido para expresar el precio del bien o servicio (sin incluir IVA).
        :param sub_tip_serv: Atributo condicional para identificar el subtipo del servicio prestado.
        :param rfc_tercero_autorizado: Atributo opcional para registrar el RFC del tercero autorizado como personal de apoyo, por quien está registrado en la plataforma tecnológica para prestar servicios o en su caso, enajenar bienes.
        :param impuestos_trasladadosdel_servicio: Nodo condicional para detallar la información de los impuestos trasladados respecto de las operaciones realizadas por personas físicas o personas morales utilizando plataformas tecnológicas.
        :param contribucion_gubernamental: Nodo opcional para detallar la información de las contribuciones gubernamentales pagadas por los servicios realizados por personas físicas utilizando plataformas tecnológicas; por ejemplo, impuesto sobre hospedaje.
        :param comision_del_servicio: Nodo condicional para detallar la información de la comisión pagada por el uso de plataformas tecnológicas por cada servicio prestado o enajenación relacionado.
        """
        
        super().__init__({
            'FormaPagoServ': forma_pago_serv,
            'TipoDeServ': tipo_de_serv,
            'FechaServ': fecha_serv,
            'PrecioServSinIVA': precio_serv_sin_iva,
            'SubTipServ': sub_tip_serv,
            'RFCTerceroAutorizado': rfc_tercero_autorizado,
            'ImpuestosTrasladadosdelServicio': impuestos_trasladadosdel_servicio,
            'ContribucionGubernamental': contribucion_gubernamental,
            'ComisionDelServicio': comision_del_servicio,
        })
        

class ServiciosPlataformasTecnologicas(CFDI):
    """
    Complemento para expresar la información sobre la enajenación de bienes y los servicios prestados por personas físicas o personas morales que utilicen plataformas tecnológicas.
    """
    tag = '{http://www.sat.gob.mx/esquemas/retencionpago/1/PlataformasTecnologicas10}ServiciosPlataformasTecnologicas'
    version = '1.0'
    
    def __init__(
            self,
            periodicidad: str,
            num_serv: int,
            mon_tot_serv_siva: Decimal | int,
            total_iva_trasladado: Decimal | int,
            total_iva_retenido: Decimal | int,
            total_isr_retenido: Decimal | int,
            dif_iva_entregado_prest_serv: Decimal | int,
            mon_totalpor_uso_plataforma: Decimal | int,
            servicios: DetallesDelServicio | dict | Sequence[DetallesDelServicio | dict],
            mon_total_contribucion_gubernamental: Decimal | int = None,
    ): 
        """
        Complemento para expresar la información sobre la enajenación de bienes y los servicios prestados por personas físicas o personas morales que utilicen plataformas tecnológicas.
        
        :param periodicidad: Atributo requerido para especificar el periodo de retención.
        :param num_serv: Atributo requerido para expresar el número de operaciones celebradas por concepto de servicios o enajenación de bienes realizadas en el periodo.
        :param mon_tot_serv_siva: Atributo requerido para expresar monto total de las operaciones realizadas en el periodo, sin incluir el monto del IVA; según corresponda.
        :param total_iva_trasladado: Atributo requerido para expresar el monto total del IVA trasladado por las operaciones realizadas en el periodo.
        :param total_iva_retenido: Atributo requerido para expresar el monto total del IVA retenido por las operaciones realizadas en el periodo.
        :param total_isr_retenido: Atributo requerido para expresar el monto total del ISR retenido por las operaciones realizadas en el periodo.
        :param dif_iva_entregado_prest_serv: Atributo requerido para expresar la diferencia del IVA entregado al prestador del servicio o enajenante de bienes en el periodo.
        :param mon_totalpor_uso_plataforma: Atributo requerido para expresar el monto total cobrado al prestador del servicio o enajenante de bienes por el uso de la plataforma en el periodo.
        :param servicios: Nodo requerido para detallar la información de operaciones celebradas por personas físicas o personas morales que utilicen plataformas tecnológicas.
        :param mon_total_contribucion_gubernamental: Atributo condicional para expresar la suma de los atributos “ImpContrib“ del nodo hijo “ContribucionGubernamental” del periodo que corresponda.
        """
        
        super().__init__({
            'Version': self.version,
            'Periodicidad': periodicidad,
            'NumServ': num_serv,
            'MonTotServSIVA': mon_tot_serv_siva,
            'TotalIVATrasladado': total_iva_trasladado,
            'TotalIVARetenido': total_iva_retenido,
            'TotalISRRetenido': total_isr_retenido,
            'DifIVAEntregadoPrestServ': dif_iva_entregado_prest_serv,
            'MonTotalporUsoPlataforma': mon_totalpor_uso_plataforma,
            'Servicios': servicios,
            'MonTotalContribucionGubernamental': mon_total_contribucion_gubernamental,
        })
        




from decimal import Decimal
from datetime import datetime, date, time
from collections.abc import Sequence
from ...cfdi import CFDI, XElement


class FinanciamientoBursatilType(XElement):
    def __init__(
            self,
            fecha_emision: date = None,
            monto_solicitado: str = None,
            monto_recibido: str = None,
    ): 
        """
        
        :param fecha_emision:
        :param monto_solicitado:
        :param monto_recibido:
        """
        
        super().__init__({
            'FechaEmision': fecha_emision,
            'MontoSolicitado': monto_solicitado,
            'MontoRecibido': monto_recibido,
        })
        

class DetalleAcreedoresType(XElement):
    def __init__(
            self,
            tipo_persona_acreedor: TipoPersonaType | dict | Sequence[TipoPersonaType | dict] = None,
    ): 
        """
        
        :param tipo_persona_acreedor:
        """
        
        super().__init__({
            'TipoPersonaAcreedor': tipo_persona_acreedor,
        })
        

class DatosPrestamoNoFinType(XElement):
    def __init__(
            self,
            detalle_acreedores: DetalleAcreedoresType | dict,
            monto_prestamo: str = None,
            moneda: str = None,
            plazo_meses: str = None,
    ): 
        """
        
        :param detalle_acreedores:
        :param monto_prestamo:
        :param moneda:
        :param plazo_meses:
        """
        
        super().__init__({
            'DetalleAcreedores': detalle_acreedores,
            'MontoPrestamo': monto_prestamo,
            'Moneda': moneda,
            'PlazoMeses': plazo_meses,
        })
        

class PrestamoNoFinancieroType(XElement):
    def __init__(
            self,
            datos_prestamo: DatosPrestamoNoFinType | dict | Sequence[DatosPrestamoNoFinType | dict] = None,
    ): 
        """
        
        :param datos_prestamo:
        """
        
        super().__init__({
            'DatosPrestamo': datos_prestamo,
        })
        

class DatosPrestamoType(XElement):
    def __init__(
            self,
            tipo_institucion: str = None,
            institucion: str = None,
            tipo_credito: str = None,
            monto_prestamo: str = None,
            moneda: str = None,
            plazo_meses: str = None,
    ): 
        """
        
        :param tipo_institucion:
        :param institucion:
        :param tipo_credito:
        :param monto_prestamo:
        :param moneda:
        :param plazo_meses:
        """
        
        super().__init__({
            'TipoInstitucion': tipo_institucion,
            'Institucion': institucion,
            'TipoCredito': tipo_credito,
            'MontoPrestamo': monto_prestamo,
            'Moneda': moneda,
            'PlazoMeses': plazo_meses,
        })
        

class PrestamoFinancieroType(XElement):
    def __init__(
            self,
            datos_prestamo: DatosPrestamoType | dict | Sequence[DatosPrestamoType | dict] = None,
    ): 
        """
        
        :param datos_prestamo:
        """
        
        super().__init__({
            'DatosPrestamo': datos_prestamo,
        })
        

class AportacionNumerarioTercerosType(XElement):
    def __init__(
            self,
            instrumento_monetario: str = None,
            moneda: str = None,
            monto_aportacion: str = None,
            aportacion_fideicomiso: str = None,
            nombre_institucion: str = None,
            valor_inmueble_preventa: str = None,
    ): 
        """
        
        :param instrumento_monetario:
        :param moneda:
        :param monto_aportacion:
        :param aportacion_fideicomiso:
        :param nombre_institucion:
        :param valor_inmueble_preventa:
        """
        
        super().__init__({
            'InstrumentoMonetario': instrumento_monetario,
            'Moneda': moneda,
            'MontoAportacion': monto_aportacion,
            'AportacionFideicomiso': aportacion_fideicomiso,
            'NombreInstitucion': nombre_institucion,
            'ValorInmueblePreventa': valor_inmueble_preventa,
        })
        

class DatosAportacionTercerosType(XElement):
    def __init__(
            self,
            aportacion_numerario: AportacionNumerarioTercerosType | dict = None,
            aportacion_especie: AportacionEspecieType | dict = None,
    ): 
        """
        
        :param aportacion_numerario:
        :param aportacion_especie:
        """
        
        super().__init__({
            'AportacionNumerario': aportacion_numerario,
            'AportacionEspecie': aportacion_especie,
        })
        

class TercerosAportacionType(XElement):
    def __init__(
            self,
            datos_aportacion: DatosAportacionTercerosType | dict,
    ): 
        """
        
        :param datos_aportacion:
        """
        
        super().__init__({
            'DatosAportacion': datos_aportacion,
        })
        

class FideicomisoType(XElement):
    def __init__(
            self,
            denominacion_razon: str = None,
            rfc: str = None,
            identificador_fideicomiso: str = None,
    ): 
        """
        
        :param denominacion_razon:
        :param rfc:
        :param identificador_fideicomiso:
        """
        
        super().__init__({
            'DenominacionRazon': denominacion_razon,
            'Rfc': rfc,
            'IdentificadorFideicomiso': identificador_fideicomiso,
        })
        

class PersonaMoralType(XElement):
    def __init__(
            self,
            representante_apoderado: RepresentanteApoderadoType | dict,
            denominacion_razon: str = None,
            fecha_constitucion: date = None,
            rfc: str = None,
            pais_nacionalidad: str = None,
            giro_mercantil: str = None,
    ): 
        """
        
        :param representante_apoderado:
        :param denominacion_razon:
        :param fecha_constitucion:
        :param rfc:
        :param pais_nacionalidad:
        :param giro_mercantil:
        """
        
        super().__init__({
            'RepresentanteApoderado': representante_apoderado,
            'DenominacionRazon': denominacion_razon,
            'FechaConstitucion': fecha_constitucion,
            'Rfc': rfc,
            'PaisNacionalidad': pais_nacionalidad,
            'GiroMercantil': giro_mercantil,
        })
        

class PersonaFisicaType(XElement):
    def __init__(
            self,
            nombre: str = None,
            apellido_paterno: str = None,
            apellido_materno: str = None,
            fecha_nacimiento: date = None,
            rfc: str = None,
            curp: str = None,
            pais_nacionalidad: str = None,
            actividad_economica: str = None,
    ): 
        """
        
        :param nombre:
        :param apellido_paterno:
        :param apellido_materno:
        :param fecha_nacimiento:
        :param rfc:
        :param curp:
        :param pais_nacionalidad:
        :param actividad_economica:
        """
        
        super().__init__({
            'Nombre': nombre,
            'ApellidoPaterno': apellido_paterno,
            'ApellidoMaterno': apellido_materno,
            'FechaNacimiento': fecha_nacimiento,
            'Rfc': rfc,
            'Curp': curp,
            'PaisNacionalidad': pais_nacionalidad,
            'ActividadEconomica': actividad_economica,
        })
        

class TipoPersonaType(XElement):
    def __init__(
            self,
            persona_fisica: PersonaFisicaType | dict = None,
            persona_moral: PersonaMoralType | dict = None,
            fideicomiso: FideicomisoType | dict = None,
    ): 
        """
        
        :param persona_fisica:
        :param persona_moral:
        :param fideicomiso:
        """
        
        super().__init__({
            'PersonaFisica': persona_fisica,
            'PersonaMoral': persona_moral,
            'Fideicomiso': fideicomiso,
        })
        

class DatosTerceroType(XElement):
    def __init__(
            self,
            tipo_persona_tercero: TipoPersonaType | dict,
            detalle_aportaciones: TercerosAportacionType | dict,
            tipo_tercero: str = None,
            descripcion_tercero: str = None,
    ): 
        """
        
        :param tipo_persona_tercero:
        :param detalle_aportaciones:
        :param tipo_tercero:
        :param descripcion_tercero:
        """
        
        super().__init__({
            'TipoPersonaTercero': tipo_persona_tercero,
            'DetalleAportaciones': detalle_aportaciones,
            'TipoTercero': tipo_tercero,
            'DescripcionTercero': descripcion_tercero,
        })
        

class DetalleTercerosType(XElement):
    def __init__(
            self,
            datos_tercero: DatosTerceroType | dict | Sequence[DatosTerceroType | dict] = None,
    ): 
        """
        
        :param datos_tercero:
        """
        
        super().__init__({
            'DatosTercero': datos_tercero,
        })
        

class TercerosType(XElement):
    def __init__(
            self,
            detalle_terceros: DetalleTercerosType | dict,
            numero_terceros: str = None,
    ): 
        """
        
        :param detalle_terceros:
        :param numero_terceros:
        """
        
        super().__init__({
            'DetalleTerceros': detalle_terceros,
            'NumeroTerceros': numero_terceros,
        })
        

class DatosAportacionSociosType(XElement):
    def __init__(
            self,
            aportacion_numerario: AportacionNumerarioType | dict = None,
            aportacion_especie: AportacionEspecieType | dict = None,
    ): 
        """
        
        :param aportacion_numerario:
        :param aportacion_especie:
        """
        
        super().__init__({
            'AportacionNumerario': aportacion_numerario,
            'AportacionEspecie': aportacion_especie,
        })
        

class SociosAportacionType(XElement):
    def __init__(
            self,
            datos_aportacion: DatosAportacionSociosType | dict,
    ): 
        """
        
        :param datos_aportacion:
        """
        
        super().__init__({
            'DatosAportacion': datos_aportacion,
        })
        

class TelefonoType(XElement):
    def __init__(
            self,
            clave_pais: str = None,
            numero_telefono: str = None,
            correo_electronico: str = None,
    ): 
        """
        
        :param clave_pais:
        :param numero_telefono:
        :param correo_electronico:
        """
        
        super().__init__({
            'ClavePais': clave_pais,
            'NumeroTelefono': numero_telefono,
            'CorreoElectronico': correo_electronico,
        })
        

class ExtranjeroType(XElement):
    def __init__(
            self,
            pais: str = None,
            estado_provincia: str = None,
            ciudad_poblacion: str = None,
            colonia: str = None,
            calle: str = None,
            numero_exterior: str = None,
            numero_interior: str = None,
            codigo_postal: str = None,
    ): 
        """
        
        :param pais:
        :param estado_provincia:
        :param ciudad_poblacion:
        :param colonia:
        :param calle:
        :param numero_exterior:
        :param numero_interior:
        :param codigo_postal:
        """
        
        super().__init__({
            'Pais': pais,
            'EstadoProvincia': estado_provincia,
            'CiudadPoblacion': ciudad_poblacion,
            'Colonia': colonia,
            'Calle': calle,
            'NumeroExterior': numero_exterior,
            'NumeroInterior': numero_interior,
            'CodigoPostal': codigo_postal,
        })
        

class NacionalType(XElement):
    def __init__(
            self,
            colonia: str = None,
            calle: str = None,
            numero_exterior: str = None,
            numero_interior: str = None,
            codigo_postal: str = None,
    ): 
        """
        
        :param colonia:
        :param calle:
        :param numero_exterior:
        :param numero_interior:
        :param codigo_postal:
        """
        
        super().__init__({
            'Colonia': colonia,
            'Calle': calle,
            'NumeroExterior': numero_exterior,
            'NumeroInterior': numero_interior,
            'CodigoPostal': codigo_postal,
        })
        

class TipoDomicilioType(XElement):
    def __init__(
            self,
            nacional: NacionalType | dict = None,
            extranjero: ExtranjeroType | dict = None,
    ): 
        """
        
        :param nacional:
        :param extranjero:
        """
        
        super().__init__({
            'Nacional': nacional,
            'Extranjero': extranjero,
        })
        

class RepresentanteApoderadoType(XElement):
    def __init__(
            self,
            nombre: str = None,
            apellido_paterno: str = None,
            apellido_materno: str = None,
            fecha_nacimiento: date = None,
            rfc: str = None,
            curp: str = None,
    ): 
        """
        
        :param nombre:
        :param apellido_paterno:
        :param apellido_materno:
        :param fecha_nacimiento:
        :param rfc:
        :param curp:
        """
        
        super().__init__({
            'Nombre': nombre,
            'ApellidoPaterno': apellido_paterno,
            'ApellidoMaterno': apellido_materno,
            'FechaNacimiento': fecha_nacimiento,
            'Rfc': rfc,
            'Curp': curp,
        })
        

class PersonaMoralSocioType(XElement):
    def __init__(
            self,
            representante_apoderado: RepresentanteApoderadoType | dict,
            denominacion_razon: str = None,
            fecha_constitucion: date = None,
            pais_nacionalidad: str = None,
            giro_mercantil: str = None,
    ): 
        """
        
        :param representante_apoderado:
        :param denominacion_razon:
        :param fecha_constitucion:
        :param pais_nacionalidad:
        :param giro_mercantil:
        """
        
        super().__init__({
            'RepresentanteApoderado': representante_apoderado,
            'DenominacionRazon': denominacion_razon,
            'FechaConstitucion': fecha_constitucion,
            'PaisNacionalidad': pais_nacionalidad,
            'GiroMercantil': giro_mercantil,
        })
        

class PersonaFisicaSocioType(XElement):
    def __init__(
            self,
            nombre: str = None,
            apellido_paterno: str = None,
            apellido_materno: str = None,
            fecha_nacimiento: date = None,
            curp: str = None,
            pais_nacionalidad: str = None,
            actividad_economica: str = None,
    ): 
        """
        
        :param nombre:
        :param apellido_paterno:
        :param apellido_materno:
        :param fecha_nacimiento:
        :param curp:
        :param pais_nacionalidad:
        :param actividad_economica:
        """
        
        super().__init__({
            'Nombre': nombre,
            'ApellidoPaterno': apellido_paterno,
            'ApellidoMaterno': apellido_materno,
            'FechaNacimiento': fecha_nacimiento,
            'Curp': curp,
            'PaisNacionalidad': pais_nacionalidad,
            'ActividadEconomica': actividad_economica,
        })
        

class TipoPersonaSocioType(XElement):
    def __init__(
            self,
            persona_fisica: PersonaFisicaSocioType | dict = None,
            persona_moral: PersonaMoralSocioType | dict = None,
            fideicomiso: FideicomisoType | dict = None,
    ): 
        """
        
        :param persona_fisica:
        :param persona_moral:
        :param fideicomiso:
        """
        
        super().__init__({
            'PersonaFisica': persona_fisica,
            'PersonaMoral': persona_moral,
            'Fideicomiso': fideicomiso,
        })
        

class DatosSocioType(XElement):
    def __init__(
            self,
            tipo_persona_socio: TipoPersonaSocioType | dict,
            tipo_domicilio_socio: TipoDomicilioType | dict,
            detalle_aportaciones: SociosAportacionType | dict,
            aportacion_anterior_socio: str = None,
            rfc_socio: str = None,
            telefono: TelefonoType | dict = None,
    ): 
        """
        
        :param tipo_persona_socio:
        :param tipo_domicilio_socio:
        :param detalle_aportaciones:
        :param aportacion_anterior_socio:
        :param rfc_socio:
        :param telefono:
        """
        
        super().__init__({
            'TipoPersonaSocio': tipo_persona_socio,
            'TipoDomicilioSocio': tipo_domicilio_socio,
            'DetalleAportaciones': detalle_aportaciones,
            'AportacionAnteriorSocio': aportacion_anterior_socio,
            'RfcSocio': rfc_socio,
            'Telefono': telefono,
        })
        

class DetalleSociosType(XElement):
    def __init__(
            self,
            datos_socio: DatosSocioType | dict | Sequence[DatosSocioType | dict] = None,
    ): 
        """
        
        :param datos_socio:
        """
        
        super().__init__({
            'DatosSocio': datos_socio,
        })
        

class SociosType(XElement):
    def __init__(
            self,
            detalle_socios: DetalleSociosType | dict,
            numero_socios: str = None,
    ): 
        """
        
        :param detalle_socios:
        :param numero_socios:
        """
        
        super().__init__({
            'DetalleSocios': detalle_socios,
            'NumeroSocios': numero_socios,
        })
        

class AportacionEspecieType(XElement):
    def __init__(
            self,
            descripcion_bien: str = None,
            monto_estimado: str = None,
    ): 
        """
        
        :param descripcion_bien:
        :param monto_estimado:
        """
        
        super().__init__({
            'DescripcionBien': descripcion_bien,
            'MontoEstimado': monto_estimado,
        })
        

class AportacionNumerarioType(XElement):
    def __init__(
            self,
            instrumento_monetario: str = None,
            moneda: str = None,
            monto_aportacion: str = None,
            aportacion_fideicomiso: str = None,
            nombre_institucion: str = None,
    ): 
        """
        
        :param instrumento_monetario:
        :param moneda:
        :param monto_aportacion:
        :param aportacion_fideicomiso:
        :param nombre_institucion:
        """
        
        super().__init__({
            'InstrumentoMonetario': instrumento_monetario,
            'Moneda': moneda,
            'MontoAportacion': monto_aportacion,
            'AportacionFideicomiso': aportacion_fideicomiso,
            'NombreInstitucion': nombre_institucion,
        })
        

class DatosAportacionType(XElement):
    def __init__(
            self,
            aportacion_numerario: AportacionNumerarioType | dict = None,
            aportacion_especie: AportacionEspecieType | dict = None,
    ): 
        """
        
        :param aportacion_numerario:
        :param aportacion_especie:
        """
        
        super().__init__({
            'AportacionNumerario': aportacion_numerario,
            'AportacionEspecie': aportacion_especie,
        })
        

class RecursosPropiosType(XElement):
    def __init__(
            self,
            datos_aportacion: DatosAportacionType | dict | Sequence[DatosAportacionType | dict] = None,
    ): 
        """
        
        :param datos_aportacion:
        """
        
        super().__init__({
            'DatosAportacion': datos_aportacion,
        })
        

class TipoAportacionType(XElement):
    def __init__(
            self,
            recursos_propios: RecursosPropiosType | dict = None,
            socios: SociosType | dict = None,
            terceros: TercerosType | dict = None,
            prestamo_financiero: PrestamoFinancieroType | dict = None,
            prestamo_no_financiero: PrestamoNoFinancieroType | dict = None,
            financiamiento_bursatil: FinanciamientoBursatilType | dict = None,
    ): 
        """
        
        :param recursos_propios:
        :param socios:
        :param terceros:
        :param prestamo_financiero:
        :param prestamo_no_financiero:
        :param financiamiento_bursatil:
        """
        
        super().__init__({
            'RecursosPropios': recursos_propios,
            'Socios': socios,
            'Terceros': terceros,
            'PrestamoFinanciero': prestamo_financiero,
            'PrestamoNoFinanciero': prestamo_no_financiero,
            'FinanciamientoBursatil': financiamiento_bursatil,
        })
        

class AportacionesType(XElement):
    def __init__(
            self,
            fecha_aportacion: date = None,
            tipo_aportacion: TipoAportacionType | dict | Sequence[TipoAportacionType | dict] = None,
    ): 
        """
        
        :param fecha_aportacion:
        :param tipo_aportacion:
        """
        
        super().__init__({
            'FechaAportacion': fecha_aportacion,
            'TipoAportacion': tipo_aportacion,
        })
        

class CaracteristicasDesarrolloType(XElement):
    def __init__(
            self,
            codigo_postal: str,
            colonia: str,
            calle: str,
            tipo_desarrollo: str,
            monto_desarrollo: str,
            unidades_comercializadas: str,
            costo_unidad: str,
            otras_empresas: str,
            descripcion_desarrollo: str = None,
    ): 
        """
        
        :param codigo_postal:
        :param colonia:
        :param calle:
        :param tipo_desarrollo:
        :param monto_desarrollo:
        :param unidades_comercializadas:
        :param costo_unidad:
        :param otras_empresas:
        :param descripcion_desarrollo:
        """
        
        super().__init__({
            'CodigoPostal': codigo_postal,
            'Colonia': colonia,
            'Calle': calle,
            'TipoDesarrollo': tipo_desarrollo,
            'MontoDesarrollo': monto_desarrollo,
            'UnidadesComercializadas': unidades_comercializadas,
            'CostoUnidad': costo_unidad,
            'OtrasEmpresas': otras_empresas,
            'DescripcionDesarrollo': descripcion_desarrollo,
        })
        

class DatosDesarrolloType(XElement):
    def __init__(
            self,
            objeto_aviso_anterior: str,
            modificacion: str,
            entidad_federativa: str,
            registro_licencia: str,
            caracteristicas_desarrollo: CaracteristicasDesarrolloType | dict | Sequence[CaracteristicasDesarrolloType | dict],
    ): 
        """
        
        :param objeto_aviso_anterior:
        :param modificacion:
        :param entidad_federativa:
        :param registro_licencia:
        :param caracteristicas_desarrollo:
        """
        
        super().__init__({
            'ObjetoAvisoAnterior': objeto_aviso_anterior,
            'Modificacion': modificacion,
            'EntidadFederativa': entidad_federativa,
            'RegistroLicencia': registro_licencia,
            'CaracteristicasDesarrollo': caracteristicas_desarrollo,
        })
        

class DesarrollosInmobiliariosType(XElement):
    def __init__(
            self,
            datos_desarrollo: DatosDesarrolloType | dict | Sequence[DatosDesarrolloType | dict],
    ): 
        """
        
        :param datos_desarrollo:
        """
        
        super().__init__({
            'DatosDesarrollo': datos_desarrollo,
        })
        

class DatosOperacionType(XElement):
    def __init__(
            self,
            tipo_operacion: str,
            desarrollos_inmobiliarios: DesarrollosInmobiliariosType | dict,
            aportaciones: AportacionesType | dict,
    ): 
        """
        
        :param tipo_operacion:
        :param desarrollos_inmobiliarios:
        :param aportaciones:
        """
        
        super().__init__({
            'TipoOperacion': tipo_operacion,
            'DesarrollosInmobiliarios': desarrollos_inmobiliarios,
            'Aportaciones': aportaciones,
        })
        

class DetalleOperacionesType(XElement):
    def __init__(
            self,
            datos_operacion: DatosOperacionType | dict | Sequence[DatosOperacionType | dict],
    ): 
        """
        
        :param datos_operacion:
        """
        
        super().__init__({
            'DatosOperacion': datos_operacion,
        })
        

class AlertaType(XElement):
    def __init__(
            self,
            tipo_alerta: str,
            descripcion_alerta: str = None,
    ): 
        """
        
        :param tipo_alerta:
        :param descripcion_alerta:
        """
        
        super().__init__({
            'TipoAlerta': tipo_alerta,
            'DescripcionAlerta': descripcion_alerta,
        })
        

class ModificatorioType(XElement):
    def __init__(
            self,
            folio_modificacion: str,
            descripcion_modificacion: str,
    ): 
        """
        
        :param folio_modificacion:
        :param descripcion_modificacion:
        """
        
        super().__init__({
            'FolioModificacion': folio_modificacion,
            'DescripcionModificacion': descripcion_modificacion,
        })
        

class AvisoType(XElement):
    def __init__(
            self,
            referencia_aviso: str,
            prioridad: str,
            alerta: AlertaType | dict,
            detalle_operaciones: DetalleOperacionesType | dict,
            modificatorio: ModificatorioType | dict = None,
    ): 
        """
        
        :param referencia_aviso:
        :param prioridad:
        :param alerta:
        :param detalle_operaciones:
        :param modificatorio:
        """
        
        super().__init__({
            'ReferenciaAviso': referencia_aviso,
            'Prioridad': prioridad,
            'Alerta': alerta,
            'DetalleOperaciones': detalle_operaciones,
            'Modificatorio': modificatorio,
        })
        

class SujetoObligadoType(XElement):
    def __init__(
            self,
            clave_sujeto_obligado: str,
            clave_actividad: str,
            clave_entidad_colegiada: str = None,
            exento: str = None,
    ): 
        """
        
        :param clave_sujeto_obligado:
        :param clave_actividad:
        :param clave_entidad_colegiada:
        :param exento:
        """
        
        super().__init__({
            'ClaveSujetoObligado': clave_sujeto_obligado,
            'ClaveActividad': clave_actividad,
            'ClaveEntidadColegiada': clave_entidad_colegiada,
            'Exento': exento,
        })
        

class InformeType(XElement):
    def __init__(
            self,
            mes_reportado: str,
            sujeto_obligado: SujetoObligadoType | dict,
            aviso: AvisoType | dict | Sequence[AvisoType | dict] = None,
    ): 
        """
        
        :param mes_reportado:
        :param sujeto_obligado:
        :param aviso:
        """
        
        super().__init__({
            'MesReportado': mes_reportado,
            'SujetoObligado': sujeto_obligado,
            'Aviso': aviso,
        })
        

class ArchivoType(XElement):
    def __init__(
            self,
            informe: InformeType | dict | Sequence[InformeType | dict],
    ): 
        """
        
        :param informe:
        """
        
        super().__init__({
            'Informe': informe,
        })
        

class Archivo(ArchivoType):
    tag = '{http://www.uif.shcp.gob.mx/recepcion/din}archivo'

